
package com.locad.geometry;

import com.locad.enumerations.Enumerations.RenderMaterialTypes;
import com.locad.enumerations.Enumerations.ShapeTypes;
import com.locad.ldraw.ColorLibrary;
import java.awt.Color;
import javax.media.opengl.GL;

public class Line extends Shape
{
    public Point3f p1 = null;
    public Point3f p2 = null;

    public Line()
    {
        super(ShapeTypes.LINE);
    }

    public static Line from_Line (String input)
    {
        Line line = new Line();

        String[] tokens = input.trim().split(" ");
        String[] t = new String[tokens.length];
        int ndx = 0;
        for (int i=0; i<tokens.length; i++)
        {
            if (tokens[i] == null ? "" != null : ! tokens[i].equals(""))
            {
                t[ndx] = tokens[i];
                ndx++;
            }
        }

        line.colorCode = new Integer(t[1]);
        line.p1 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
        line.p2 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));

        return line;
    }

    @Override
    public void addToGL(GL gl, Matrix4f orientation, int requestedColorCode, RenderMaterialTypes renderMaterialType, ColorLibrary colorLib, boolean BFCflag)
    {
        Color color = getColor(requestedColorCode, renderMaterialType, colorLib);
        if (color != null)
        {
            colorLib.getColor(requestedColorCode).material.ApplyToGL(gl);

            gl.glBegin(GL.GL_LINES);
            gl.glColor4f(color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, color.getAlpha() / 255f);
            Point3f temp = orientation.times(p1);
            gl.glVertex3f(temp.x, temp.y, temp.z);
            temp = orientation.times(p2);
            gl.glVertex3f(temp.x, temp.y, temp.z);
            gl.glEnd();
        }
    }

    @Override
    public Point3f getFurthestPointFromOrigin ()
    {
        Point3f tmp = p1;
        if (p2.mag() > tmp.mag())
            tmp = p2;

        return tmp;
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append(super.toString()).append("\n");
        strBuffer.append("p1 ").append(p1).append("\n");
        strBuffer.append("p2 ").append(p2).append("\n");

        return strBuffer.toString();
    }
}