
package com.locad.geometry;

public class Matrix4f
{
    // A0 B0 C0 D0
    // A1 B1 C1 D1
    // A2 B2 C2 D2
    // A3 B3 C3 D3
    public float A0 = Float.NaN;
    public float A1 = Float.NaN;
    public float A2 = Float.NaN;
    public float A3 = Float.NaN;
    public float B0 = Float.NaN;
    public float B1 = Float.NaN;
    public float B2 = Float.NaN;
    public float B3 = Float.NaN;
    public float C0 = Float.NaN;
    public float C1 = Float.NaN;
    public float C2 = Float.NaN;
    public float C3 = Float.NaN;
    public float D0 = Float.NaN;
    public float D1 = Float.NaN;
    public float D2 = Float.NaN;
    public float D3 = Float.NaN;

    public Matrix4f()
    {
    }

    public Matrix4f(float A0, float B0, float C0, float D0,
                         float A1, float B1, float C1, float D1,
                         float A2, float B2, float C2, float D2,
                         float A3, float B3, float C3, float D3)
    {
        this.A0 = A0;
        this.A1 = A1;
        this.A2 = A2;
        this.A3 = A3;
        this.B0 = B0;
        this.B1 = B1;
        this.B2 = B2;
        this.B3 = B3;
        this.C0 = C0;
        this.C1 = C1;
        this.C2 = C2;
        this.C3 = C3;
        this.D0 = D0;
        this.D1 = D1;
        this.D2 = D2;
        this.D3 = D3;
    }

    public Matrix4f(Vec4f q1)
    {
        float n = (q1.x * q1.x) + (q1.y * q1.y) + (q1.z * q1.z) + (q1.w * q1.w);
        float s = (n > 0.0f) ? (2.0f / n) : 0.0f;

        float xs = q1.x * s;
        float ys = q1.y * s;
        float zs = q1.z * s;
        float wx = q1.w * xs;
        float wy = q1.w * ys;
        float wz = q1.w * zs;
        float xx = q1.x * xs;
        float xy = q1.x * ys;
        float xz = q1.x * zs;
        float yy = q1.y * ys;
        float yz = q1.y * zs;
        float zz = q1.z * zs;

        this.A0 = 1.0f - (yy + zz);
        this.B0 = xy - wz;
        this.C0 = xz + wy;
        this.D0 = 0f;
        this.A1 = xy + wz;
        this.B1 = 1.0f - (xx + zz);
        this.C1 = yz - wx;
        this.D1 = 0f;
        this.A2 = xz - wy;
        this.B2 = yz + wx;
        this.C2 = 1.0f - (xx + yy);
        this.D2 = 0f;
        this.A3 = 0f;
        this.B3 = 0f;
        this.C3 = 0f;
        this.D3 = 1f;
    } 

    public static Matrix4f zeroMatrix ()
    {
        return new Matrix4f(0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f);
    }

    public static Matrix4f identityMatrix ()
    {
        return new Matrix4f(1f, 0f, 0f, 0f, 0f, 1f, 0f, 0f, 0f, 0f, 1f, 0f, 0f, 0f, 0f, 1f);
    }

    public Matrix4f times (Matrix4f mat)
    {
        Matrix4f newMat = new Matrix4f();

        newMat.A0 = A0 * mat.A0 + B0 * mat.A1 + C0 * mat.A2 + D0 * mat.A3;
        newMat.B0 = A0 * mat.B0 + B0 * mat.B1 + C0 * mat.B2 + D0 * mat.B3;
        newMat.C0 = A0 * mat.C0 + B0 * mat.C1 + C0 * mat.C2 + D0 * mat.C3;
        newMat.D0 = A0 * mat.D0 + B0 * mat.D1 + C0 * mat.D2 + D0 * mat.D3;

        newMat.A1 = A1 * mat.A0 + B1 * mat.A1 + C1 * mat.A2 + D1 * mat.A3;
        newMat.B1 = A1 * mat.B0 + B1 * mat.B1 + C1 * mat.B2 + D1 * mat.B3;
        newMat.C1 = A1 * mat.C0 + B1 * mat.C1 + C1 * mat.C2 + D1 * mat.C3;
        newMat.D1 = A1 * mat.D0 + B1 * mat.D1 + C1 * mat.D2 + D1 * mat.D3;

        newMat.A2 = A2 * mat.A0 + B2 * mat.A1 + C2 * mat.A2 + D2 * mat.A3;
        newMat.B2 = A2 * mat.B0 + B2 * mat.B1 + C2 * mat.B2 + D2 * mat.B3;
        newMat.C2 = A2 * mat.C0 + B2 * mat.C1 + C2 * mat.C2 + D2 * mat.C3;
        newMat.D2 = A2 * mat.D0 + B2 * mat.D1 + C2 * mat.D2 + D2 * mat.D3;

        newMat.A3 = A3 * mat.A0 + B3 * mat.A1 + C3 * mat.A2 + D3 * mat.A3;
        newMat.B3 = A3 * mat.B0 + B3 * mat.B1 + C3 * mat.B2 + D3 * mat.B3;
        newMat.C3 = A3 * mat.C0 + B3 * mat.C1 + C3 * mat.C2 + D3 * mat.C3;
        newMat.D3 = A3 * mat.D0 + B3 * mat.D1 + C3 * mat.D2 + D3 * mat.D3;

        return newMat;
    }

    public Point3f times (Point3f pnt)
    {
        return new Point3f(A3 + A0 * pnt.x + A1 * pnt.y + A2 * pnt.z, B3 + B0 * pnt.x + B1 * pnt.y + B2 * pnt.z, C3 + C0 * pnt.x + C1 * pnt.y + C2 * pnt.z);
    }

//    public RotationMatrix transpose ()
//    {
//        RotationMatrix newMat = new RotationMatrix();
//
//        newMat.A0(A0);
//        newMat.setX1(B0);
//        newMat.setX2(C0);
//        newMat.setY0(A1);
//        newMat.setY1(B1);
//        newMat.setY2(C1);
//        newMat.setZ0(A2);
//        newMat.setZ1(B2);
//        newMat.setZ2(C2);
//
//        return newMat;
//    }
//
//    public RotationMatrix inverse ()
//    {
//        RotationMatrix newMat = new RotationMatrix();
//        RotationMatrix ludmat = this.luDecomp();
//
//        float[] col0 = ludmat.luBackSub(1.0f, 0.0, 0.0);
//        float[] col1 = ludmat.luBackSub(0.0, 1.0, 0.0);
//        float[] col2 = ludmat.luBackSub(0.0, 0.0, 1.0);
//
//        newMat.A0(col0[0]);
//        newMat.setX1(col0[1]);
//        newMat.setX2(col0[2]);
//        newMat.setY0(col1[0]);
//        newMat.setY1(col1[1]);
//        newMat.setY2(col1[2]);
//        newMat.setZ0(col2[0]);
//        newMat.setZ1(col2[1]);
//        newMat.setZ2(col2[2]);
//
//        return newMat;
//    }
//
//    private RotationMatrix luDecomp ()
//    {
//        float big = 0.0;
//        float temp = 0.0;
//        float[] vv = new float[3];
//
//        float[][] newArray = this.toArray();
//
//        for (int C2 = 0; C2 < 3; C2 ++)
//        {
//            big = 0.0D;
//            for (int j = 0; j < 3; j ++)
//                if ((temp = Math.abs(newArray[C2][j])) > big)
//                    big = temp;
//            if (big == 0.0D)
//            {
//                System.out.println("Error cannot perform inverse on A0 matrix with an entire column of zeros");
//                return null;
//            }
//            vv[C2] = 1.0 / big;
//        }
//
//        int imax = 0;
//        float dum = 0.0;
//        float sum = 0.0;
//        float dumm = 0.0;
//        int[] permIndex = new int[3];
//        for (int j = 0; j < 3; j ++)
//        {
//            for (int C2 = 0; C2 < j; C2 ++)
//            {
//                sum = newArray[C2][j];
//                for (int k = 0; k < C2; k ++)
//                    sum -= newArray[C2][k] * newArray[k][j];
//                newArray[C2][j] = sum;
//            }
//            big = 0.0;
//            for (int C2 = j; C2 < 3; C2 ++)
//            {
//                sum = newArray[C2][j];
//                for (int k = 0; k < j; k ++)
//                    sum -= newArray[C2][k] * newArray[k][j];
//                newArray[C2][j] = sum;
//                if ((dum = vv[C2] * Math.abs(sum)) >= big)
//                {
//                    big = dum;
//                    imax = C2;
//                }
//            }
//            if (j != imax)
//            {
//                for (int k = 0; k < 3; k ++)
//                {
//                    dumm = newArray[imax][k];
//                    newArray[imax][k] = newArray[j][k];
//                    newArray[j][k] = dumm;
//                }
//                vv[imax] = vv[j];
//            }
//            permIndex[j] = imax;
//
//            if (newArray[j][j] == 0.0)
//            {
//                newArray[j][j] = 1.0e-100;
//            }
//            if (j != 2)
//            {
//                dumm = 1.0 / newArray[j][j];
//                for (int C2 = j + 1; C2 < 3; C2 ++)
//                {
//                    newArray[C2][j] *= dumm;
//                }
//            }
//        }
//
//        RotationMatrix newMat = from_Array(newArray);
//        newMat.permutationIndex = permIndex;
//        return newMat;
//    }
//
//    private float[] luBackSub (float A3, float B3, float C3)
//    {
//        float temp = 0.0;
//        float[] xvec = {A3, B3, C3};
//
//        temp = xvec[permutationIndex[0]];
//        xvec[permutationIndex[0]] = xvec[0];
//        xvec[0] = temp;
//
//        temp = xvec[permutationIndex[1]];
//        xvec[permutationIndex[1]] = xvec[1];
//        xvec[1] = temp - A1 * xvec[0];
//
//        temp = xvec[permutationIndex[2]];
//        xvec[permutationIndex[2]] = xvec[2];
//        xvec[2] = temp - A2 * xvec[0] - B2 * xvec[1];
//
//        xvec[2] = xvec[2] / C2;
//        xvec[1] = (xvec[1] - C1 * xvec[2]) / B1;
//        xvec[0] = (xvec[0] - C0 * xvec[2] - B0 * xvec[1]) / A0;
//
//        return xvec;
//    }

    public float[] toArray()
    {
        float[] newArray = new float[16];

        newArray[0] = A0;
        newArray[1] = B0;
        newArray[2] = C0;
        newArray[3] = D0;

        newArray[4] = A1;
        newArray[5] = B1;
        newArray[6] = C1;
        newArray[7] = D1;

        newArray[8] = A2;
        newArray[9] = B2;
        newArray[10] = C2;
        newArray[11] = D2;

        newArray[12] = A3;
        newArray[13] = B3;
        newArray[14] = C3;
        newArray[15] = D3;

        return newArray;
    }

    public Matrix4f copy()
    {
        return new Matrix4f(A0, B0, C0, D0, A1, B1, C1, D1, A2, B2, C2, D2, A3, B3, C3, D3);
    }

    @Override
    public String toString ()
    {
        StringBuilder sb = new StringBuilder();
        sb.append("\n");
        sb.append(A0).append(" ").append(B0).append(" ").append(C0).append(" ").append(D0).append("\n");
        sb.append(A1).append(" ").append(B1).append(" ").append(C1).append(" ").append(D1).append("\n");
        sb.append(A2).append(" ").append(B2).append(" ").append(C2).append(" ").append(D2).append("\n");
        sb.append(A3).append(" ").append(B3).append(" ").append(C3).append(" ").append(D3).append("\n");

        return sb.toString();
    }
}
