
package com.locad.geometry;

import com.locad.enumerations.Enumerations.RenderMaterialTypes;
import com.locad.enumerations.Enumerations.ShapeTypes;
import com.locad.ldraw.ColorLibrary;
import java.awt.Color;
import javax.media.opengl.GL;

public class Quadrilateral extends Shape
{
    public Point3f p1 = null;
    public Point3f p2 = null;
    public Point3f p3 = null;
    public Point3f p4 = null;
    public Point3f norm = null;

    public Quadrilateral()
    {
        super(ShapeTypes.QUADRILATERAL);
    }

    public static Quadrilateral from_Line (String input)
    {
        Quadrilateral quad = new Quadrilateral();

        String[] tokens = input.trim().split(" ");
        String[] t = new String[tokens.length];
        int ndx = 0;
        for (int i=0; i<tokens.length; i++)
        {
            if (tokens[i] == null ? "" != null : ! tokens[i].equals(""))
            {
                t[ndx] = tokens[i];
                ndx++;
            }
        }

        quad.colorCode = new Integer(t[1]);
        quad.p1 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
        quad.p2 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));
        quad.p3 = new Point3f(new Float(t[8]), new Float(t[9]), new Float(t[10]));
        quad.p4 = new Point3f(new Float(t[11]), new Float(t[12]), new Float(t[13]));

        quad.norm = getNormal(quad.p1, quad.p2, quad.p3);

        // sometimes, the quadrilateral points are not in the correct order
        double signedArea123 = getSignedArea(quad.p1, quad.p2, quad.p3, quad.norm);
        if (signedArea123 > 0)
        {
            double signedArea134 = getSignedArea(quad.p1, quad.p3, quad.p4, quad.norm);      
            if (signedArea134 > 0)
            {
                // no swap needed for clockwise order       
            }
            else
            {
                double signedArea124 = getSignedArea(quad.p1, quad.p2, quad.p4, quad.norm);
                
                if (signedArea124 > 0)
                {
                    Point3f temp = quad.p3.copy();
                    quad.p3 = quad.p4.copy();
                    quad.p4 = temp;                  
                }
                else
                {
                    Point3f temp = quad.p1.copy();
                    quad.p1 = quad.p4.copy();
                    quad.p4 = temp;            
                }
            }    
        }
        else
        {
            double signedArea134 = getSignedArea(quad.p1, quad.p3, quad.p4, quad.norm);
            if (signedArea134 < 0)
            {
                Point3f temp = quad.p2.copy();
                quad.p2 = quad.p4.copy();
                quad.p4 = temp;      
            }
            else
            {
                double signedArea124 = getSignedArea(quad.p1, quad.p2, quad.p4, quad.norm);
                if (signedArea124 > 0)
                {
                    Point3f temp = quad.p2.copy();
                    quad.p2 = quad.p3.copy();
                    quad.p3 = temp;                  
                }
                else
                {
                    Point3f temp = quad.p1.copy();
                    quad.p1 = quad.p2.copy();
                    quad.p2 = temp;            
                }
            }             
        }
        
        return quad;
    }

    private static Point3f getNormal(Point3f p1, Point3f p2, Point3f p3)
    {
        // Calculate The Vector From Point 1 To Point 0
        Point3f v1 = new Point3f(p1.getX() - p2.getX(), p1.getY() - p2.getY(), p1.getZ() - p2.getZ());

        // Calculate The Vector From Point 2 To Point 1
        Point3f v2 = new Point3f(p2.getX() - p3.getX(), p2.getY() - p3.getY(), p2.getZ() - p3.getZ());

        // Compute The Cross Product To Give Us A Surface Normal
        return v1.crossProd(v2).unitVector();
    }

    private static double getSignedArea(Point3f p1, Point3f p2, Point3f p3, Point3f normal)
    {
        // select largest abs coordinate to ignore for projection
        float ax = (normal.x > 0 ? normal.x : -normal.x);     // abs x-coord
        float ay = (normal.y > 0 ? normal.y : -normal.y);     // abs y-coord
        float az = (normal.z > 0 ? normal.z : -normal.z);     // abs z-coord
        double an = Math.sqrt(ax * ax + ay * ay + az * az);  // length of normal vector

        // project onto 2D plane and calculate area
        if (ax > ay)
        {
            if (ax > az)
            {
                return ((p2.y * (p3.z - p1.z)) + (p3.y * (p1.z - p2.z)) + (p1.y * (p2.z - p3.z))) * (an / (2 * ax));
            }
        }
        else if (ay > az)
        {
            return ((p2.x * (p3.z - p1.z)) + (p3.x * (p1.z - p2.z)) + (p1.x * (p2.z - p3.z))) * (an / (2 * ay));
        }

        return ((p2.x * (p3.y - p1.y)) + (p3.x * (p1.y - p2.y)) + (p1.x * (p2.y - p3.y))) * (an / (2 * az));
    }

    @Override
    public void addToGL(GL gl, Matrix4f orientation, int requestedColorCode, RenderMaterialTypes renderMaterialType, ColorLibrary colorLib, boolean BFCflag)
    {
        Color color = getColor(requestedColorCode, renderMaterialType, colorLib);
        if (color != null)
        {
            colorLib.getColor(requestedColorCode).material.ApplyToGL(gl);

            if (BFCflag)
            {
                gl.glBegin(GL.GL_POLYGON);
                gl.glColor4f(color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, color.getAlpha() / 255f);
                gl.glNormal3f(p1.x - norm.x, p1.y - norm.y, p1.z - norm.z);
                Point3f temp = orientation.times(p4);
                gl.glVertex3f(temp.x, temp.y, temp.z);
                temp = orientation.times(p3);
                gl.glVertex3f(temp.x, temp.y, temp.z);
                temp = orientation.times(p2);
                gl.glVertex3f(temp.x, temp.y, temp.z);
                temp = orientation.times(p1);
                gl.glVertex3f(temp.x, temp.y, temp.z);
                gl.glEnd();
             
//                gl.glBegin(GL.GL_LINES);
//                gl.glColor3f(0f, 0f, 1f);
//                gl.glVertex3f(p1.x, p1.y, p1.z);
//                gl.glVertex3f(p1.x - norm.x, p1.y - norm.y, p1.z - norm.z);
//                gl.glEnd();
            }
            else
            {
                gl.glBegin(GL.GL_POLYGON);
                gl.glColor4f(color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, color.getAlpha() / 255f);
                gl.glNormal3f(p1.x + norm.x, p1.y + norm.y, p1.z + norm.z);
                Point3f temp = orientation.times(p1);
                gl.glVertex3f(temp.x, temp.y, temp.z);
                temp = orientation.times(p2);
                gl.glVertex3f(temp.x, temp.y, temp.z);
                temp = orientation.times(p3);
                gl.glVertex3f(temp.x, temp.y, temp.z);
                temp = orientation.times(p4);
                gl.glVertex3f(temp.x, temp.y, temp.z);
                gl.glEnd();

//                gl.glBegin(GL.GL_LINES);
//                gl.glColor3f(0f, 0f, 1f);
//                gl.glVertex3f(p1.x, p1.y, p1.z);
//                gl.glVertex3f(p1.x + norm.x, p1.y + norm.y, p1.z + norm.z);
//                gl.glEnd();
            }
        }
    }

    @Override
    public Point3f getFurthestPointFromOrigin ()
    {
        Point3f tmp = p1;
        if (p2.mag() > tmp.mag())
            tmp = p2;
        if (p3.mag() > tmp.mag())
            tmp = p3;
        if (p4.mag() > tmp.mag())
            tmp = p4;

        return tmp;
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append(super.toString()).append("\n");
        strBuffer.append("p1 ").append(p1).append("\n");
        strBuffer.append("p2 ").append(p2).append("\n");
        strBuffer.append("p3 ").append(p3).append("\n");
        strBuffer.append("p4 ").append(p4).append("\n");

        return strBuffer.toString();
    }
}