
package com.locad.ldraw;

import com.locad.enumerations.Enumerations.MaterialTypes;
import java.awt.Color;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;
import org.apache.log4j.Logger;

public class ColorLibrary
{
    private final Logger log;
    
    public Map<Integer, LDrawColor> colors = new HashMap<Integer, LDrawColor>();

    public ColorLibrary (String configFilePath)
    {
        log = Logger.getLogger(getClass().getName());

        try
        {
            Scanner s = new Scanner(new BufferedReader(new FileReader(configFilePath)));

            while (s.hasNextLine())
            {
                String[] tokens = s.nextLine().trim().split(" ");
                String[] t = new String[tokens.length];
                int ndx = 0;
                for (int i=0; i<tokens.length; i++)
                {
                    if (tokens[i] == null ? "" != null : ! tokens[i].equals(""))
                    {
                        t[ndx] = tokens[i];
                        ndx++;
                    }
                }
                if (t.length >= 9)
                {
                    if (t[0].equals("0") && t[1].equals("!COLOUR"))
                    {
                        Color edgeColor;
                        Color fillColor;
                        int r, g, b;
                        int eColor;
                        int luminance;
                        MaterialTypes materialType = MaterialTypes.PLASTIC;
                        if (t[ 8].startsWith("#"))
                        {
                            eColor = Integer.valueOf(t[8].substring(1), 16);
                            r = (eColor >> 16) & 0xFF;
                            g = (eColor >> 8) & 0xFF;
                            b = eColor & 0xFF;
                            edgeColor = new Color(r, g, b);
                        }
                        else
                        {
                            edgeColor = new Color(0, 0, 0);
                        }
                        eColor = Integer.valueOf(t[6].substring(1), 16);
                        r = (eColor >> 16) & 0xFF;
                        g = (eColor >> 8) & 0xFF;
                        b = eColor & 0xFF;
                        fillColor = new Color(r, g, b);
                        luminance = 0;
                        for (int i=9; i<ndx; i++)
                        {
                            if (ndx >= i+1)
                            {
                                if (t[i].equalsIgnoreCase("ALPHA"))
                                {
                                    fillColor = new Color(r, g, b, Integer.valueOf(t[10]));
                                }
                                if (t[i].equalsIgnoreCase("LUMINANCE"))
                                {
                                    luminance = Integer.valueOf(t[12]);
                                }
                                if (t[i].equalsIgnoreCase("RUBBER"))
                                {
                                    materialType = MaterialTypes.RUBBER;
                                }
                                if (t[i].equalsIgnoreCase("METAL"))
                                {
                                    materialType = MaterialTypes.METAL;
                                }
                                if (t[i].equalsIgnoreCase("PEARLESCENT"))
                                {
                                    materialType = MaterialTypes.PEARLESCENT;
                                }
                                if (t[i].equalsIgnoreCase("CHROME"))
                                {
                                    materialType = MaterialTypes.CHROME;
                                }
                            }
                        }
                        colors.put(Integer.valueOf(t[4]), new LDrawColor(t[2], fillColor, edgeColor, luminance, materialType));
                    }
                }
            }
            s.close();
        }
        catch (FileNotFoundException ex)
        {
            log.error("error " + ex);
        }
    }

    public LDrawColor getColor (int code)
    {
        if (colors.containsKey(code))
        {
            return colors.get(code);
        }
        else
        {
            for (Integer key : colors.keySet())
            {
                if (colors.get(key).name.equalsIgnoreCase("WHITE"))
                {
                    return colors.get(key);
                }
            }
            return null;
        }
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        for (Integer key : colors.keySet())
        {
            strBuffer.append("ColorCode: ").append(key).append(colors.get(key).toString());
        }

        return strBuffer.toString();
    }
}
