
package com.locad.ldraw;

import com.locad.enumerations.Enumerations.MaterialTypes;
import com.locad.enumerations.Enumerations.RenderMaterialTypes;
import com.locad.viewport.Material;
import java.awt.Color;

public class LDrawColor
{
    public final String name;
    public final Color fillColor;
    public final Color edgeColor;
    public final int luminance;
    public final MaterialTypes materialType;
    public final RenderMaterialTypes renderMaterialType;
    public final Material material;

    public LDrawColor (String name, Color fillColor, Color edgeColor, int luminance, MaterialTypes materialType)
    {
        this.name = name.toUpperCase();
        this.fillColor = fillColor;
        this.edgeColor = edgeColor;
        this.luminance = luminance;
        this.materialType = materialType;

        if (luminance > 0 && fillColor.getAlpha() > 250)
            renderMaterialType = RenderMaterialTypes.LUMINANT;
        else if(fillColor.getAlpha() < 250)
            renderMaterialType = RenderMaterialTypes.TRANSPARENT;
        else if (materialType == MaterialTypes.CHROME)
            renderMaterialType = RenderMaterialTypes.REFLECTIVE;
        else
            renderMaterialType = RenderMaterialTypes.ABSORBANT;

        if (materialType == MaterialTypes.RUBBER)
            material = new Material(new float[] {0.2f, 0.2f, 0.2f, 1.0f}, new float[] {-0.2f, -0.2f, -0.2f, 1.0f}, new float[] {-0.2f, -0.2f, -0.2f, 1.0f}, 0.0f);
        else if (materialType == MaterialTypes.PEARLESCENT)
            material = new Material(new float[] {0.1f, 0.1f, 0.1f, 1.0f}, new float[] {0.7f, 0.7f, 0.7f, 1.0f}, new float[] {0.7f, 0.7f, 0.7f, 1.0f}, 0.2f);
        else if (materialType == MaterialTypes.METAL)
            material = new Material(new float[] {0.3f, 0.3f, 0.3f, 1.0f}, new float[] {0.8f, 0.8f, 0.8f, 1.0f}, new float[] {0.8f, 0.8f, 0.8f, 1.0f}, 0.2f);
        else if (materialType == MaterialTypes.CHROME)
            material = new Material(new float[] {0.4f, 0.4f, 0.4f, 1.0f}, new float[] {0.9f, 0.9f, 0.9f, 1.0f}, new float[] {0.9f, 0.9f, 0.9f, 1.0f}, 0.4f);
        else
            material = new Material(new float[] {0.0f, 0.0f, 0.0f, 1.0f}, new float[] {0.5f, 0.5f, 0.5f, 1.0f}, new float[] {0.7f, 0.7f, 0.7f, 1.0f}, 0.1f);
    }
    
    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append(" name: ").append(name).append(" fillColor: ").append(fillColor).append(" edgeColor: ").append(edgeColor).append(" luminance: ").append(luminance).append("\n");

        return strBuffer.toString();
    }
}