
package com.locad.ldraw;

import com.locad.enumerations.Enumerations.ShapeTypes;
import com.locad.geometry.*;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import org.apache.log4j.Logger;

public class Part extends Shape
{
    private final Logger log;

    public String fileName = "";

    public List<Shape> shapes = new ArrayList<Shape>();

    public boolean official = true;
    public Point3f origin = new Point3f(0f, 0f, 0f);
    public Matrix4f orientation = Matrix4f.identityMatrix();

    public Part ()
    {
        super(ShapeTypes.PART);
        log = Logger.getLogger(getClass().getName());
    }

    public static Part from_File (String fileName)
    {
        boolean BFCflag = false;

        Part part = new Part();
        part.fileName = fileName.replace("\\", "/").toUpperCase();

        try
        {
            Scanner s = new Scanner(new BufferedReader(new FileReader(fileName)));

            while (s.hasNextLine())
            {
                String line = s.nextLine();
                String[] tokens = line.trim().split(" ");
                String[] t = new String[tokens.length];
                int ndx = 0;
                for (int j=0; j<tokens.length; j++)
                {
                    if (tokens[j] == null ? "" != null : !tokens[j].equals(""))
                    {
                        t[ndx] = tokens[j];
                        ndx++;
                    }
                }
                if (t.length > 1)
                {
                    if(t[0].equals("0") && t[1].equalsIgnoreCase("BFC"))
                    {
                        BFCflag = true;
                    }
                    else if(t[0].equals("1"))
                    {
                        part.shapes.add(PartEmbeded.from_Line(line, BFCflag));
                        BFCflag = false;
                    }
                    else if(t[0].equals("2"))
                    {
                        part.shapes.add(Line.from_Line(line));
                        BFCflag = false;
                    }
                    else if (t[0].equals("3"))
                    {
                        part.shapes.add(Triangle.from_Line(line));
                        BFCflag = false;
                    }
                    else if (t[0].equals("4"))
                    {
                        part.shapes.add(Quadrilateral.from_Line(line));
                        BFCflag = false;
                    }
                    else if (t[0].equals("5"))
                    {
                        //part.shapes.add(LineConditional.from_Line(line));
                        BFCflag = false;
                    }
                }
            }
            s.close();
        }
        catch (FileNotFoundException ex)
        {
            part.log.error("error " + ex);
        }

        //part.log.info(part);
        return part;
    }

//    @Override
//    public void addToGL(GL gl, int requestedColorCode, ColorLibrary colorLib)
//    {
//        for (Shape prim : shapes)
//        {
//            if (prim instanceof PartEmbeded)
//            {
//                addToGL(partLib.getPrimitive(((PartEmbeded) prim).fileName), ((PartEmbeded) prim).origin, ((PartEmbeded) prim).orientation, requestedColorCode, gl);
//            }
//            else
//                prim.addToGL(gl, requestedColorCode, colorLib);
//        }
//    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append("fileName ").append(fileName).append("\n");
        //strBuffer.append("official ").append(official).append("\n");
        strBuffer.append("origin ").append(origin.toString()).append("\n");
        //strBuffer.append("orientation ").append(orientation.toString()).append("\n");

//        for (Shape line : shapes)
//        {
//            strBuffer.append(line.toString()).append("\n");
//        }

        return strBuffer.toString();
    }
}

