
package com.locad.ldraw;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import org.apache.log4j.Logger;

public class PartsLibrary
{
    private final Logger log;

    public final String ldrawDir;
    private Map<String, Part> parts = new HashMap<String, Part>();
    public Map<String, String> partPaths = new HashMap<String, String>();

//    private static PartsLibrary instance = null;
//
//    public static PartsLibrary getInstance ()
//    {
//        synchronized (PartsLibrary.class)
//        {
//            if (instance == null)
//            {
//                System.out.println("Creating Instance of PartsLibrary");
//                instance = new PartsLibrary();
//            }
//        }
//        return instance;
//    }

    public PartsLibrary (String ldrawDir)
    {
        log = Logger.getLogger(getClass().getName());

        this.ldrawDir = ldrawDir;

        //scanDir(partPaths, ldrawDir + "MODELS/SUB_MODELS/", "");
        //scanDir(partPaths, ldrawDir + "MODELS/", "");
        scanDir(partPaths, ldrawDir + "PARTS/", "");
        scanDir(partPaths, ldrawDir + "PARTS/", "S/");
        scanDir(partPaths, ldrawDir + "P/", "");
        scanDir(partPaths, ldrawDir + "P/", "48/");
    }

    private void scanDir (Map<String, String> partPaths, String folder, String prefix)
    {
        File dir = new File(folder + prefix);
        String[] chld = dir.list();
        if (chld != null)
        {
            for (int i = 0; i < chld.length; i ++)
            {
                String filePath = folder + prefix + chld[i];
                String fileName = (prefix + chld[i]).replace("\\", "/").toUpperCase();
                if (partPaths.get(fileName) != null)
                {
                    log.warn("Duplicate part file found: " + filePath + " and " + partPaths.get(fileName).toString());
                }
                else
                {
                    //log.info("fileName: " + fileName + " filePath: " + filePath);
                    partPaths.put(fileName, filePath);
                }
            }
        }
    }

    public Part getPart(String primitiveName)
    {
        primitiveName = primitiveName.replace("\\", "/").toUpperCase();

        if (parts.get(primitiveName) != null)
        {
            return parts.get(primitiveName);
        }
        else
        {
            if (partPaths.containsKey(primitiveName))
            {
                //log.info("creating new primitiveName: " + primitiveName);
                parts.put(primitiveName, Part.from_File(partPaths.get(primitiveName)));
                return parts.get(primitiveName);
            }
        }
        log.warn("not found in list primitiveName: " + primitiveName);
        return null;
    }

    public boolean fileExists (String filename)
    {
        filename = filename.replace("/", File.pathSeparator);

        File file = new File(filename);

        return file.exists();
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append("ldrawDir ").append(ldrawDir).append("\n");

        for (String name : partPaths.keySet())
        {
            strBuffer.append(name).append(" ").append(partPaths.get(name).toString()).append("\n");
        }

        for (String name : partPaths.keySet())
        {
            strBuffer.append(name).append(" ").append(partPaths.get(name).toString()).append("\n");
        }

        return strBuffer.toString();
    }
}
