
package com.locad.main;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;
import org.apache.log4j.Appender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

public class LoggingUtils
{
    public static final File DEFAULT_LOGGER_CONFIG_FILE = new File("config/logger.xml");
    public static final File DEFAULT_BACKUP_LOGGER_CONFIG_FILE = new File("logger.xml");
    public static int DEFAULT_PERIODICITY = 5000; // milliseconds
    public static String dateBasedLogDirectory;

    @SuppressWarnings ("unchecked")
    public static void setupLog4j (String logFileName)
    {
        // Read the env variable for the log4j properties file and load it
        String log4jFileName = System.getProperty("log4j.configuration");

        // get the default log config file
        File configFile = null;
        if (log4jFileName != null && log4jFileName.length() > 0)
        {
            configFile = new File(log4jFileName);
            if ( ! configFile.exists())
            {
                configFile = DEFAULT_LOGGER_CONFIG_FILE;
            }
        }
        else
        {
            configFile = DEFAULT_LOGGER_CONFIG_FILE;
        }

        if ( ! configFile.exists())
        {
            configFile = DEFAULT_BACKUP_LOGGER_CONFIG_FILE;
        }

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy_MM_dd__HH_mm");
        String dateDirectoryName = dateFormat.format(new Date());

        // if the config file _still_ doesn't exist, setup a default console output and complain
        if ( ! configFile.exists())
        {
            System.out.println("The logger config file was not specified correctly");
        }
        else
        {
            DOMConfigurator.configureAndWatch(configFile.toString(),
                    DEFAULT_PERIODICITY);
            if (logFileName != null && logFileName.length() > 0)
            {
                // get the file logger and change the file name property
                Logger rootLogger = Logger.getRootLogger();
                Enumeration appenders = rootLogger.getAllAppenders();
                while (appenders.hasMoreElements())
                {
                    Appender appender = (Appender) appenders.nextElement();
                    if (FileAppender.class.isAssignableFrom(
                            appender.getClass()))
                    {
                        FileAppender fileAppender = (FileAppender) appender;
                        String logFileString = logFileName;
                        File currentLogFile = new File(fileAppender.getFile());
                        File oldLogFile = null;
                        if (currentLogFile.isFile())
                        {
                            File logFile = new File(
                                    new File(currentLogFile.getParentFile(),
                                    dateDirectoryName), logFileName);
                            dateBasedLogDirectory = logFile.getParentFile().toString();
                            if ( ! logFile.getParentFile().exists())
                            {
                                logFile.getParentFile().mkdirs();
                            }
                            logFileString = logFile.toString();
                            oldLogFile = currentLogFile;
                        }
                        fileAppender.setFile(logFileString);
                        fileAppender.activateOptions();
                        if (oldLogFile != null)
                        {
                            if (oldLogFile.exists())
                            {
                                oldLogFile.delete();
                            }
                        }
                    }
                }
            }
        }

        // adjust the directory where the DDS data gets logged to a new subdirectory named according to the current date and time
        Set<Appender> allAppenders = new HashSet<Appender>();
        Enumeration loggers = LogManager.getCurrentLoggers();
        while (loggers.hasMoreElements())
        {
            Logger logger = (Logger) loggers.nextElement();
            Enumeration appenders = logger.getAllAppenders();
            while (appenders.hasMoreElements())
            {
                allAppenders.add((Appender) appenders.nextElement());
            }
        }

        // remove the root appender(s) since it should be addressed by the code above
        allAppenders.removeAll(Collections.list(
                Logger.getRootLogger().getAllAppenders()));

        for (Appender appender : allAppenders)
        {
            if (FileAppender.class.isAssignableFrom(appender.getClass()))
            {
                FileAppender fa = (FileAppender) appender;
                File originalFileSpec = new File(fa.getFile());
                File originalDirectory = originalFileSpec.getParentFile();

                // note: you can adjust the directory name here
                File newDirectory = new File(originalDirectory,
                        dateDirectoryName);
                File newFileSpec = new File(newDirectory,
                        originalFileSpec.getName());
                fa.setFile(newFileSpec.toString());

                // activate our changes
                fa.activateOptions();

                // try to delete the original file, if it exists
                if (originalFileSpec.exists())
                {
                    originalFileSpec.delete();
                }
            }
        }
    }
}
