
package com.locad.main;

import com.locad.ldraw.ColorLibrary;
import com.locad.ldraw.Part;
import com.locad.ldraw.PartsLibrary;
import com.locad.viewport.Renderer;
import com.sun.opengl.util.FPSAnimator;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLCapabilities;
import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

public class Main extends JFrame
{
    private static final Logger log = Logger.getLogger(Main.class);

    public GLCanvas glCanvas;
    public FPSAnimator animator;

    private static final int defaultColorCode = 14;
    private static final Color bgColor = Color.WHITE;
    private static final String ldrawDir = "C:/LDraw/";
    private static final String colorFile = ldrawDir + "LDConfig.ldr";

    private static final String partFile = ldrawDir + "parts/54872p01.DAT";
    //private static final String partFile = ldrawDir + "parts/3005pt0.DAT";
    //private static final String partFile = ldrawDir + "parts/3001.DAT";

    public Main()
    {
        init();

        ColorLibrary colorLib = new ColorLibrary(colorFile);
        PartsLibrary partLib = new PartsLibrary(ldrawDir);

        Part part = Part.from_File(partFile);

        setTitle(part.fileName);

        // set color code of new part
        part.colorCode = defaultColorCode;

        // set origin of new part
        part.origin.x = 0f;
        part.origin.y = 0f;
        part.origin.z = 0f;

        // set orientation of new part
        //part.orientation.setX0(1.0);

        glCanvas = new GLCanvas(createGLCapabilites());
        glCanvas.addGLEventListener(new Renderer(part, partLib, colorLib, bgColor));

        add(glCanvas, java.awt.BorderLayout.CENTER);
        animator = new FPSAnimator(glCanvas, 30);

        this.addWindowListener(new WindowAdapter()
        {
            @Override
            public void windowClosing(WindowEvent e)
            {
                // Run this on another thread than the AWT event queue to make sure the call to Animator.stop() completes before exiting
                new Thread(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        animator.stop();
                        System.exit(0);
                    }
                }).start();
            }
        });

        animator.start();
        glCanvas.requestFocus();
    }

    private void init()
    {
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        this.setPreferredSize(new Dimension(500, 500));
        this.setSize(new Dimension(500, 500));

//        GroupLayout panelLayout = new GroupLayout(panel);
//        panel.setLayout(panelLayout);
//        panelLayout.setHorizontalGroup(panelLayout.createParallelGroup(Alignment.LEADING).addGap(0, 380, Short.MAX_VALUE));
//        panelLayout.setVerticalGroup(panelLayout.createParallelGroup(Alignment.LEADING).addGap(0, 258, Short.MAX_VALUE));
//
//        GroupLayout layout = new GroupLayout(getContentPane());
//        getContentPane().setLayout(layout);
//        layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING).addGroup(layout.createSequentialGroup().addContainerGap().addGroup(layout.createParallelGroup(Alignment.LEADING).addComponent(panel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addComponent(label)).addContainerGap()));
//        layout.setVerticalGroup(layout.createParallelGroup(Alignment.LEADING).addGroup(layout.createSequentialGroup().addContainerGap().addComponent(label).addPreferredGap(ComponentPlacement.RELATED).addComponent(panel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addContainerGap()));

        //pack();
    }

    @Override
    public void setVisible(boolean show)
    {
        if (!show)
        {
            animator.stop();
        }
        super.setVisible(show);
        if (!show)
        {
            animator.start();
        }
    }

    private GLCapabilities createGLCapabilites()
    {

        GLCapabilities capabilities = new GLCapabilities();
        capabilities.setHardwareAccelerated(true);

        // try to enable 2x anti aliasing - should be supported on most hardware
        capabilities.setNumSamples(2);
        capabilities.setSampleBuffers(true);

        return capabilities;
    }

    public static void main(String[] args)
    {
        DOMConfigurator.configureAndWatch("config/log4j.xml", 500);
        //LoggingUtils.setupLog4j("LOCAD.log");

        // Run this in the AWT event thread to prevent deadlocks and race conditions
        EventQueue.invokeLater(new Runnable()
        {
            @Override
            public void run()
            {
                // switch to system l&f for native font rendering etc.
                try
                {
                    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                }
                catch (Exception ex)
                {
                    log.warn("can not enable system look and feel " + ex);
                }

                Main frame = new Main();
                frame.setVisible(true);
            }
        });
    }
}
