
package com.locad.viewport;

import java.awt.event.*;

public class Listener implements KeyListener, MouseListener, MouseMotionListener, MouseWheelListener
{
    private final Scene scene;
    private final Camera camera;

    public Listener (Scene scene, Camera camera)
    {
        this.scene = scene;
        this.camera = camera;
    }

    public void keyPressed(KeyEvent e)
    {
        if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
        {
            System.exit(0);     // ESC for exit
        }
    }

    public void keyReleased(KeyEvent e)
    {
    }

    public void keyTyped(KeyEvent e)
    {
    }

    public void mouseEntered(MouseEvent e)
    {
    }

    public void mouseExited(MouseEvent e)
    {
    }

    public void mousePressed(MouseEvent e)
    {
        scene.setPickPoint(e.getPoint(), true);
        camera.startDrag(e.getPoint());
    }

    public void mouseReleased(MouseEvent e)
    {
    }

    public void mouseClicked(MouseEvent e)
    {
        scene.setPickPoint(e.getPoint(), true);
        camera.reset();
    }

    public void mouseDragged(MouseEvent e)
    {
        // Button 1 pressed
        if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
        {
            camera.rotate(e.getPoint());
        }

        // Button 2 pressed
        if ((e.getModifiers() & MouseEvent.BUTTON2_MASK) != 0)
        {
            camera.pan(e.getPoint());
        }
    }

    public void mouseWheelMoved (MouseWheelEvent e)
    {
        if (e.getScrollType() == MouseWheelEvent.WHEEL_UNIT_SCROLL)
        {
            camera.zoom(e.getScrollAmount(), e.getWheelRotation());
        }
    }

    // Passive motion callback to capture mouse movements while buttons are not pressed
    public void mouseMoved(MouseEvent e)
    {
        scene.setPickPoint(e.getPoint(), false);
    }
}