
package com.locad.viewport;

import com.locad.enumerations.Enumerations.RenderMaterialTypes;
import com.locad.geometry.Matrix4f;
import com.locad.geometry.Point3f;
import com.locad.geometry.Shape;
import com.locad.ldraw.ColorLibrary;
import com.locad.ldraw.Part;
import com.locad.ldraw.PartEmbeded;
import com.locad.ldraw.PartsLibrary;
import java.awt.Point;
import javax.media.opengl.GL;
import org.apache.log4j.Logger;

public class Scene
{
    private final Logger log;

    private Part part;
    private final PartsLibrary partLib;
    private final ColorLibrary colorLib;

    public int scene_dl;

    public Scene(Part part, PartsLibrary partLib, ColorLibrary colorLib)
    {
        log = Logger.getLogger(getClass().getName());

        this.part = part;
        this.partLib = partLib;
        this.colorLib = colorLib;
    }

    public void createScene(GL gl)
    {
        scene_dl = gl.glGenLists(1);
        gl.glNewList(scene_dl, GL.GL_COMPILE);

        opaqueMaterial(gl);

        // render luminant objects first
        render(part, part.orientation, part.colorCode, RenderMaterialTypes.LUMINANT, false, gl);

        // render absorbant objects second
        render(part, part.orientation, part.colorCode, RenderMaterialTypes.ABSORBANT, false, gl);

        // render reflective objects third
        render(part, part.orientation, part.colorCode, RenderMaterialTypes.REFLECTIVE, false, gl);

        // render transparent objects last
        transparentMaterial(gl);
        render(part, part.orientation, part.colorCode, RenderMaterialTypes.TRANSPARENT, false, gl);

        gl.glEndList();
    }

    private void luminantMaterial(GL gl)
    {
        //gl.glMaterialfv(GL.GL_FRONT, GL.GL_EMISSION, blankMaterial, 0);
    }

    private void opaqueMaterial(GL gl)
    {
        gl.glEnable(GL.GL_LIGHTING);
        gl.glDisable(GL.GL_BLEND);
        gl.glEnable(GL.GL_COLOR_MATERIAL);
    }

    private void transparentMaterial(GL gl)
    {
        gl.glDisable(GL.GL_LIGHTING);
        gl.glEnable(GL.GL_BLEND);
        gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
    }

    public void render (Part part, Matrix4f orientation, int requestedColorCode, RenderMaterialTypes renderMaterialType, boolean BFCflag, GL gl)
    {
        gl.glLineWidth(1.0f);
        for (Shape prim : part.shapes)
        {
            if (prim instanceof PartEmbeded)
            {
                PartEmbeded p = (PartEmbeded) prim;
                if (((PartEmbeded) prim).colorCode == 16)
                    render(partLib.getPart(p.fileName), p.orientation.times(orientation), requestedColorCode, renderMaterialType, p.BFCflag, gl);
                else
                    render(partLib.getPart(p.fileName), p.orientation.times(orientation), p.colorCode, renderMaterialType, p.BFCflag, gl);
            }
            else
            {
                prim.addToGL(gl, orientation, requestedColorCode, renderMaterialType, colorLib, BFCflag);
            }
        }
    }

    // compute bounding box
    public Point3f getFurthestPointFromOrigin (Part part, Matrix4f orientation)
    {
        float x = 0f;
        float y = 0f;
        float z = 0f;
        Point3f temp = new Point3f(0f, 0f, 0f);

        if (part == null)
            return temp;

        for (Shape prim : part.shapes)
        {
            if (prim instanceof PartEmbeded)
            {
                if (((PartEmbeded) prim).colorCode == 16)
                    temp = getFurthestPointFromOrigin(partLib.getPart(((PartEmbeded) prim).fileName), ((PartEmbeded) prim).orientation);
                else
                    temp = getFurthestPointFromOrigin(partLib.getPart(((PartEmbeded) prim).fileName), ((PartEmbeded) prim).orientation);

                temp = orientation.times(temp);

                if (Math.abs(temp.x) > x)
                    x = Math.abs(temp.x);
                if (Math.abs(temp.y) > y)
                    y = Math.abs(temp.y);
                if (Math.abs(temp.z) > z)
                    z = Math.abs(temp.z);
            }
            else
            {
                temp = prim.getFurthestPointFromOrigin();
                temp = orientation.times(temp);
                if (Math.abs(temp.x) > x)
                    x = Math.abs(temp.x);
                if (Math.abs(temp.y) > y)
                    y = Math.abs(temp.y);
                if (Math.abs(temp.z) > z)
                    z = Math.abs(temp.z);
            }
        }

        return new Point3f(x, y, z);
    }

    public void setPickPoint(Point point, boolean mouseDown)
    {
    }
}
