
package com.locad.geometry;

import com.locad.enumerations.Enumerations.MaterialTypes;
import com.locad.enumerations.Enumerations.ShapeTypes;
import com.locad.enumerations.Enumerations.ViewAngleModes;
import java.util.ArrayList;
import java.util.Map;
import javax.media.opengl.GL;

public class Line extends Shape
{
    public float thickness = 0;
    public Point3f p1 = null;
    public Point3f p2 = null;

    public Line()
    {
        super(ShapeTypes.LINE);
    }

    public static Line from_Line (String input, Matrix4f orientation, boolean BFCFlag)
    {
        Line shape = new Line();

        String[] tokens = input.trim().split(" ");
        String[] t = new String[tokens.length];
        int ndx = 0;
        for (int i=0; i<tokens.length; i++)
        {
            if (tokens[i] == null ? "" != null : ! tokens[i].equals(""))
            {
                t[ndx] = tokens[i];
                ndx++;
            }
        }

        shape.colorCode = new Integer(t[1]);

        if (BFCFlag)
        {
            shape.p1 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            shape.p2 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));
        }
        else
        {
            shape.p2 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            shape.p1 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));
        }

        // apply orientation from parent
        shape.p1 = orientation.times(shape.p1);
        shape.p2 = orientation.times(shape.p2);

        return shape;
    }

    @Override
    public void setOrientation(Matrix4f orientation)
    {
        p1 = orientation.times(p1);
        p2 = orientation.times(p2);
    }

    @Override
    public void addToGL(GL gl)
    {
        gl.glBegin(GL.GL_LINES);
        gl.glColor4f(color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, color.getAlpha() / 255f);
        gl.glVertex3f(p1.x, p1.y, p1.z);
        gl.glVertex3f(p2.x, p2.y, p2.z);
        gl.glEnd();
    }

    @Override
    public void addToGLwithBFC(GL gl)
    {
        gl.glBegin(GL.GL_LINES);
        gl.glColor4f(color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, color.getAlpha() / 255f);
        gl.glNormal3f(1, 0, 0);
        gl.glVertex3f(p1.x, p1.y, p1.z);
        gl.glVertex3f(p2.x, p2.y, p2.z);
        gl.glEnd();
    }

    @Override
    public void addToList(Map<MaterialTypes, ArrayList<Shape>> materials, int requestedColorCode, float lineWidth)
    {
        setColor(requestedColorCode, this.colorCode);

        thickness = lineWidth;

        for (MaterialTypes materialType : material.materialTypes)
            materials.get(materialType).add(this);
    }
    
    @Override
    public void addToListBFC(Map<MaterialTypes, ArrayList<Shape>> materials, int requestedColorCode, float lineWidth, boolean BFCflag)
    {
        setColor(requestedColorCode, this.colorCode);

        if (!BFCflag)
        {
            Point3f temp = p1.copy();
            p1 = p2;
            p2 = temp;
        }

        thickness = lineWidth;

        for (MaterialTypes materialType : material.materialTypes)
            materials.get(materialType).add(this);
    }

    @Override
    public Point3f getCentroid (Point3f min, Point3f max)
    {
        minPoint = new Point3f(Float.MAX_VALUE, Float.MAX_VALUE, Float.MAX_VALUE);
        maxPoint = new Point3f(-Float.MAX_VALUE, -Float.MAX_VALUE, -Float.MAX_VALUE);

        if (p1.x < minPoint.x)
            minPoint.x = p1.x;
        if (p1.x > maxPoint.x)
            maxPoint.x = p1.x;
        if (p1.y < minPoint.y)
            minPoint.y = p1.y;
        if (p1.y > maxPoint.y)
            maxPoint.y = p1.y;
        if (p1.z < minPoint.z)
            minPoint.z = p1.z;
        if (p1.z > maxPoint.z)
            maxPoint.z = p1.z;

        if (p2.x < minPoint.x)
            minPoint.x = p2.x;
        if (p2.x > maxPoint.x)
            maxPoint.x = p2.x;
        if (p2.y < minPoint.y)
            minPoint.y = p2.y;
        if (p2.y > maxPoint.y)
            maxPoint.y = p2.y;
        if (p2.z < minPoint.z)
            minPoint.z = p2.z;
        if (p2.z > maxPoint.z)
            maxPoint.z = p2.z;

        if (minPoint.x < min.x)
            min.x = minPoint.x;
        if (maxPoint.x > max.x)
            max.x = maxPoint.x;
        if (minPoint.y < min.y)
            min.y = minPoint.y;
        if (maxPoint.y > max.y)
            max.y = maxPoint.y;
        if (minPoint.z < min.z)
            min.z = minPoint.z;
        if (maxPoint.z > max.z)
            max.z = maxPoint.z; 

        centroid = new Point3f(maxPoint.x - (maxPoint.x - minPoint.x) / 2f, maxPoint.y - (maxPoint.y - minPoint.y) / 2f, maxPoint.z - (maxPoint.z - minPoint.z) / 2f);

        return centroid;
    }

    @Override
    public boolean contains (float x, float y, ViewAngleModes viewAngle)
    {
        float t = thickness / 2f;
        Point3f q1 = new Point3f(p1.x - t, p1.y - t, p1.z - t);
        Point3f q2 = new Point3f(p2.x - t, p2.y - t, p2.z - t);
        Point3f q3 = new Point3f(p2.x + t, p2.y + t, p2.z + t);
        Point3f q4 = new Point3f(p1.x + t, p1.y + t, p1.z + t);

        int crossings = 0;

        if(viewAngle == ViewAngleModes.FRONT || viewAngle == ViewAngleModes.BACK)
        {
            if (((q1.y <= y) && (y < q2.y) || (q2.y <= y) && (y < q1.y)) && x < ((q2.x - q1.x) / (q2.y - q1.y)) * (y - q1.y) + q1.x)
                crossings++;
            if (((q2.y <= y) && (y < q3.y) || (q3.y <= y) && (y < q2.y)) && x < ((q3.x - q2.x) / (q3.y - q2.y)) * (y - q2.y) + q2.x)
                crossings++;
            if (((q3.y <= y) && (y < q4.y) || (q4.y <= y) && (y < q3.y)) && x < ((q4.x - q3.x) / (q4.y - q3.y)) * (y - q3.y) + q3.x)
                crossings++;
            if (((q4.y <= y) && (y < q1.y) || (q1.y <= y) && (y < q4.y)) && x < ((q1.x - q4.x) / (q1.y - q4.y)) * (y - q4.y) + q4.x)
                crossings++;
        }
        else if(viewAngle == ViewAngleModes.LEFT || viewAngle == ViewAngleModes.RIGHT)
        {
            if (((q1.y <= y) && (y < q2.y) || (q2.y <= y) && (y < q1.y)) && x < ((q2.z - q1.z) / (q2.y - q1.y)) * (y - q1.y) + q1.z)
                crossings++;
            if (((q2.y <= y) && (y < q3.y) || (q3.y <= y) && (y < q2.y)) && x < ((q3.z - q2.z) / (q3.y - q2.y)) * (y - q2.y) + q2.z)
                crossings++;
            if (((q3.y <= y) && (y < q4.y) || (q4.y <= y) && (y < q3.y)) && x < ((q4.z - q3.z) / (q4.y - q3.y)) * (y - q3.y) + q3.z)
                crossings++;
            if (((q4.y <= y) && (y < q1.y) || (q1.y <= y) && (y < q4.y)) && x < ((q1.z - q4.z) / (q1.y - q4.y)) * (y - q4.y) + q4.z)
                crossings++;
        }
        else if(viewAngle == ViewAngleModes.TOP || viewAngle == ViewAngleModes.BOTTOM)
        {
            if (((q1.z <= y) && (y < q2.z) || (q2.z <= y) && (y < q1.z)) && x < ((q2.x - q1.x) / (q2.z - q1.z)) * (y - q1.z) + q1.x)
                crossings++;
            if (((q2.z <= y) && (y < q3.z) || (q3.z <= y) && (y < q2.z)) && x < ((q3.x - q2.x) / (q3.z - q2.z)) * (y - q2.z) + q2.x)
                crossings++;
            if (((q3.z <= y) && (y < q4.z) || (q4.z <= y) && (y < q3.z)) && x < ((q4.x - q3.x) / (q4.z - q3.z)) * (y - q3.z) + q3.x)
                crossings++;
            if (((q4.z <= y) && (y < q1.z) || (q1.z <= y) && (y < q4.z)) && x < ((q1.x - q4.x) / (q1.z - q4.z)) * (y - q4.z) + q4.x)
                crossings++;
        }

        if (crossings % 2 != 0)
        {
            return true;
        }
        return false;
    }

    @Override
    public Line copy()
    {
        Line that = new Line();

        that.color = this.color;
        that.colorCode = this.colorCode;
        if (this.material != null)
            that.material = this.material.copy();
        that.centroid = this.centroid.copy();
        that.minPoint = this.minPoint.copy();
        that.maxPoint = this.maxPoint.copy();

        that.p1 = this.p1.copy();
        that.p2 = this.p2.copy();

        return that;
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append(super.toString()).append("\n");
        strBuffer.append("p1 ").append(p1).append("\n");
        strBuffer.append("p2 ").append(p2).append("\n");

        return strBuffer.toString();
    }
}