
package com.locad.geometry;

import com.locad.enumerations.Enumerations.MaterialTypes;
import com.locad.enumerations.Enumerations.ShapeTypes;
import com.locad.enumerations.Enumerations.ViewAngleModes;
import java.util.ArrayList;
import java.util.Map;
import javax.media.opengl.GL;

public class Quadrilateral extends Shape
{
    public Point3f p1 = null;
    public Point3f p2 = null;
    public Point3f p3 = null;
    public Point3f p4 = null;
    public Point3f norm = null;

    public Quadrilateral()
    {
        super(ShapeTypes.QUADRILATERAL);
    }

    public static Quadrilateral from_Line (String input, Matrix4f orientation, boolean BFCFlag)
    {
        Quadrilateral shape = new Quadrilateral();

        String[] tokens = input.trim().split(" ");
        String[] t = new String[tokens.length];
        int ndx = 0;
        for (int i=0; i<tokens.length; i++)
        {
            if (tokens[i] == null ? "" != null : ! tokens[i].equals(""))
            {
                t[ndx] = tokens[i];
                ndx++;
            }
        }

        shape.colorCode = new Integer(t[1]);

        if (BFCFlag)
        {
            shape.p4 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            shape.p3 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));
            shape.p2 = new Point3f(new Float(t[8]), new Float(t[9]), new Float(t[10]));
            shape.p1 = new Point3f(new Float(t[11]), new Float(t[12]), new Float(t[13]));
        }
        else
        {
            shape.p1 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            shape.p2 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));
            shape.p3 = new Point3f(new Float(t[8]), new Float(t[9]), new Float(t[10]));
            shape.p4 = new Point3f(new Float(t[11]), new Float(t[12]), new Float(t[13]));
        }

        // apply orientation from parent
        shape.p1 = orientation.times(shape.p1);
        shape.p2 = orientation.times(shape.p2);
        shape.p3 = orientation.times(shape.p3);
        shape.p4 = orientation.times(shape.p4);

        shape.norm = getNormal(shape.p1, shape.p2, shape.p3);

        // sometimes, the shaperilateral points are not in the correct order
        double signedArea123 = getSignedArea(shape.p1, shape.p2, shape.p3, shape.norm);
        if (signedArea123 > 0)
        {
            double signedArea134 = getSignedArea(shape.p1, shape.p3, shape.p4, shape.norm);      
            if (signedArea134 > 0)
            {
                // no swap needed for clockwise order       
            }
            else
            {
                double signedArea124 = getSignedArea(shape.p1, shape.p2, shape.p4, shape.norm);
                
                if (signedArea124 > 0)
                {
                    Point3f temp = shape.p3.copy();
                    shape.p3 = shape.p4.copy();
                    shape.p4 = temp;                  
                }
                else
                {
                    Point3f temp = shape.p1.copy();
                    shape.p1 = shape.p4.copy();
                    shape.p4 = temp;            
                }
            }    
        }
        else
        {
            double signedArea134 = getSignedArea(shape.p1, shape.p3, shape.p4, shape.norm);
            if (signedArea134 < 0)
            {
                Point3f temp = shape.p2.copy();
                shape.p2 = shape.p4.copy();
                shape.p4 = temp;      
            }
            else
            {
                double signedArea124 = getSignedArea(shape.p1, shape.p2, shape.p4, shape.norm);
                if (signedArea124 > 0)
                {
                    Point3f temp = shape.p2.copy();
                    shape.p2 = shape.p3.copy();
                    shape.p3 = temp;                  
                }
                else
                {
                    Point3f temp = shape.p1.copy();
                    shape.p1 = shape.p2.copy();
                    shape.p2 = temp;            
                }
            }             
        }
        
        return shape;
    }

    private static Point3f getNormal(Point3f p1, Point3f p2, Point3f p3)
    {
        // Calculate The Vector From Point 1 To Point 0
        Point3f v1 = new Point3f(p1.getX() - p2.getX(), p1.getY() - p2.getY(), p1.getZ() - p2.getZ());

        // Calculate The Vector From Point 2 To Point 1
        Point3f v2 = new Point3f(p2.getX() - p3.getX(), p2.getY() - p3.getY(), p2.getZ() - p3.getZ());

        // Compute The Cross Product To Give Us A Surface Normal
        return v1.crossProd(v2).unitVector();
    }

    private static double getSignedArea(Point3f p1, Point3f p2, Point3f p3, Point3f normal)
    {
        // select largest abs coordinate to ignore for projection
        float ax = (normal.x > 0 ? normal.x : -normal.x);     // abs x-coord
        float ay = (normal.y > 0 ? normal.y : -normal.y);     // abs y-coord
        float az = (normal.z > 0 ? normal.z : -normal.z);     // abs z-coord
        double an = Math.sqrt(ax * ax + ay * ay + az * az);  // length of normal vector

        // project onto 2D plane and calculate area
        if (ax > ay)
        {
            if (ax > az)
            {
                return ((p2.y * (p3.z - p1.z)) + (p3.y * (p1.z - p2.z)) + (p1.y * (p2.z - p3.z))) * (an / (2 * ax));
            }
        }
        else if (ay > az)
        {
            return ((p2.x * (p3.z - p1.z)) + (p3.x * (p1.z - p2.z)) + (p1.x * (p2.z - p3.z))) * (an / (2 * ay));
        }

        return ((p2.x * (p3.y - p1.y)) + (p3.x * (p1.y - p2.y)) + (p1.x * (p2.y - p3.y))) * (an / (2 * az));
    }

    @Override
    public void setOrientation(Matrix4f orientation)
    {
        p1 = orientation.times(p1);
        p2 = orientation.times(p2);
        p3 = orientation.times(p3);
        p4 = orientation.times(p4);
    }

    @Override
    public void addToGL(GL gl)
    {
        gl.glBegin(GL.GL_QUADS);
        gl.glColor4f(color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, color.getAlpha() / 255f);
        gl.glVertex3f(p1.x, p1.y, p1.z);
        gl.glVertex3f(p2.x, p2.y, p2.z);
        gl.glVertex3f(p3.x, p3.y, p3.z);
        gl.glVertex3f(p4.x, p4.y, p4.z);
        gl.glEnd();
    }

    @Override
    public void addToGLwithBFC(GL gl)
    {
        gl.glBegin(GL.GL_QUADS);
        gl.glColor4f(color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, color.getAlpha() / 255f);
        gl.glNormal3f(norm.x, norm.y, norm.z);
        gl.glVertex3f(p1.x, p1.y, p1.z);
        gl.glVertex3f(p2.x, p2.y, p2.z);
        gl.glVertex3f(p3.x, p3.y, p3.z);
        gl.glVertex3f(p4.x, p4.y, p4.z);
        gl.glEnd();
    }

    @Override
    public void addToList(Map<MaterialTypes, ArrayList<Shape>> materials, int requestedColorCode, float lineWidth)
    {
        setColor(requestedColorCode, this.colorCode);

        for (MaterialTypes materialType : material.materialTypes)
            materials.get(materialType).add(this);
    }

    @Override
    public void addToListBFC(Map<MaterialTypes, ArrayList<Shape>> materials, int requestedColorCode, float lineWidth, boolean BFCflag)
    {
        setColor(requestedColorCode, this.colorCode);
        
        if (BFCflag)
        {
            Point3f temp = p1.copy();
            p1 = p4;
            p4 = temp;

            temp = p2.copy();
            p2 = p3;
            p3 = temp;
        }

        for (MaterialTypes materialType : material.materialTypes)
            materials.get(materialType).add(this);
    }

    @Override
    public Point3f getCentroid (Point3f min, Point3f max)
    {
        minPoint = new Point3f(Float.MAX_VALUE, Float.MAX_VALUE, Float.MAX_VALUE);
        maxPoint = new Point3f(-Float.MAX_VALUE, -Float.MAX_VALUE, -Float.MAX_VALUE);

        if (p1.x < minPoint.x)
            minPoint.x = p1.x;
        if (p1.x > maxPoint.x)
            maxPoint.x = p1.x;
        if (p1.y < minPoint.y)
            minPoint.y = p1.y;
        if (p1.y > maxPoint.y)
            maxPoint.y = p1.y;
        if (p1.z < minPoint.z)
            minPoint.z = p1.z;
        if (p1.z > maxPoint.z)
            maxPoint.z = p1.z;

        if (p2.x < minPoint.x)
            minPoint.x = p2.x;
        if (p2.x > maxPoint.x)
            maxPoint.x = p2.x;
        if (p2.y < minPoint.y)
            minPoint.y = p2.y;
        if (p2.y > maxPoint.y)
            maxPoint.y = p2.y;
        if (p2.z < minPoint.z)
            minPoint.z = p2.z;
        if (p2.z > maxPoint.z)
            maxPoint.z = p2.z;

        if (p3.x < minPoint.x)
            minPoint.x = p3.x;
        if (p3.x > maxPoint.x)
            maxPoint.x = p3.x;
        if (p3.y < minPoint.y)
            minPoint.y = p3.y;
        if (p3.y > maxPoint.y)
            maxPoint.y = p3.y;
        if (p3.z < minPoint.z)
            minPoint.z = p3.z;
        if (p3.z > maxPoint.z)
            maxPoint.z = p3.z;

        if (p4.x < minPoint.x)
            minPoint.x = p1.x;
        if (p4.x > maxPoint.x)
            maxPoint.x = p4.x;
        if (p4.y < minPoint.y)
            minPoint.y = p4.y;
        if (p4.y > maxPoint.y)
            maxPoint.y = p4.y;
        if (p4.z < minPoint.z)
            minPoint.z = p4.z;
        if (p4.z > maxPoint.z)
            maxPoint.z = p4.z;

        if (minPoint.x < min.x)
            min.x = minPoint.x;
        if (maxPoint.x > max.x)
            max.x = maxPoint.x;
        if (minPoint.y < min.y)
            min.y = minPoint.y;
        if (maxPoint.y > max.y)
            max.y = maxPoint.y;
        if (minPoint.z < min.z)
            min.z = minPoint.z;
        if (maxPoint.z > max.z)
            max.z = maxPoint.z;

        centroid = new Point3f(maxPoint.x - (maxPoint.x - minPoint.x) / 2f, maxPoint.y - (maxPoint.y - minPoint.y) / 2f, maxPoint.z - (maxPoint.z - minPoint.z) / 2f);

        return centroid;
    }

    @Override
    public boolean contains (float x, float y, ViewAngleModes viewAngle)
    {
        int crossings = 0;
        if(viewAngle == ViewAngleModes.FRONT || viewAngle == ViewAngleModes.BACK)
        {
            if (((p1.y <= y) && (y < p2.y) || (p2.y <= y) && (y < p1.y)) && x < ((p2.x - p1.x) / (p2.y - p1.y)) * (y - p1.y) + p1.x)
                crossings++;
            if (((p2.y <= y) && (y < p3.y) || (p3.y <= y) && (y < p2.y)) && x < ((p3.x - p2.x) / (p3.y - p2.y)) * (y - p2.y) + p2.x)
                crossings++;
            if (((p3.y <= y) && (y < p4.y) || (p4.y <= y) && (y < p3.y)) && x < ((p4.x - p3.x) / (p4.y - p3.y)) * (y - p3.y) + p3.x)
                crossings++;
            if (((p4.y <= y) && (y < p1.y) || (p1.y <= y) && (y < p4.y)) && x < ((p1.x - p4.x) / (p1.y - p4.y)) * (y - p4.y) + p4.x)
                crossings++;
        }
        else if(viewAngle == ViewAngleModes.LEFT || viewAngle == ViewAngleModes.RIGHT)
        {
            if (((p1.y <= y) && (y < p2.y) || (p2.y <= y) && (y < p1.y)) && x < ((p2.z - p1.z) / (p2.y - p1.y)) * (y - p1.y) + p1.z)
                crossings++;
            if (((p2.y <= y) && (y < p3.y) || (p3.y <= y) && (y < p2.y)) && x < ((p3.z - p2.z) / (p3.y - p2.y)) * (y - p2.y) + p2.z)
                crossings++;
            if (((p3.y <= y) && (y < p4.y) || (p4.y <= y) && (y < p3.y)) && x < ((p4.z - p3.z) / (p4.y - p3.y)) * (y - p3.y) + p3.z)
                crossings++;
            if (((p4.y <= y) && (y < p1.y) || (p1.y <= y) && (y < p4.y)) && x < ((p1.z - p4.z) / (p1.y - p4.y)) * (y - p4.y) + p4.z)
                crossings++;
        }
        else if(viewAngle == ViewAngleModes.TOP || viewAngle == ViewAngleModes.BOTTOM)
        {
            if (((p1.z <= y) && (y < p2.z) || (p2.z <= y) && (y < p1.z)) && x < ((p2.x - p1.x) / (p2.z - p1.z)) * (y - p1.z) + p1.x)
                crossings++;
            if (((p2.z <= y) && (y < p3.z) || (p3.z <= y) && (y < p2.z)) && x < ((p3.x - p2.x) / (p3.z - p2.z)) * (y - p2.z) + p2.x)
                crossings++;
            if (((p3.z <= y) && (y < p4.z) || (p4.z <= y) && (y < p3.z)) && x < ((p4.x - p3.x) / (p4.z - p3.z)) * (y - p3.z) + p3.x)
                crossings++;
            if (((p4.z <= y) && (y < p1.z) || (p1.z <= y) && (y < p4.z)) && x < ((p1.x - p4.x) / (p1.z - p4.z)) * (y - p4.z) + p4.x)
                crossings++;
        }

        if (crossings % 2 != 0)
        {
            return true;
        }
        return false;
    }

    @Override
    public Quadrilateral copy()
    {
        Quadrilateral that = new Quadrilateral();

        that.color = this.color;
        that.colorCode = this.colorCode;
        if (this.material != null)
            that.material = this.material.copy();
        that.centroid = this.centroid.copy();
        that.minPoint = this.minPoint.copy();
        that.maxPoint = this.maxPoint.copy();

        that.norm = this.norm.copy();
        that.p1 = this.p1.copy();
        that.p2 = this.p2.copy();
        that.p3 = this.p3.copy();
        that.p4 = this.p4.copy();

        return that;
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append(super.toString()).append("\n");
        strBuffer.append("p1 ").append(p1).append("\n");
        strBuffer.append("p2 ").append(p2).append("\n");
        strBuffer.append("p3 ").append(p3).append("\n");
        strBuffer.append("p4 ").append(p4).append("\n");

        return strBuffer.toString();
    }
}