
package com.locad.geometry;

import com.locad.enumerations.Enumerations.MaterialTypes;
import com.locad.enumerations.Enumerations.ShapeTypes;
import com.locad.enumerations.Enumerations.ViewAngleModes;
import java.util.ArrayList;
import java.util.Map;
import javax.media.opengl.GL;

public class Triangle extends Shape
{
    public Point3f p1 = null;
    public Point3f p2 = null;
    public Point3f p3 = null;
    public Point3f norm = null;

    public Triangle()
    {
        super(ShapeTypes.TRIANGLE);
    }

    public static Triangle from_Line (String input, Matrix4f orientation, boolean BFCFlag)
    {
        Triangle shape = new Triangle();

        String[] tokens = input.trim().split(" ");
        String[] t = new String[tokens.length];
        int ndx = 0;
        for (int i=0; i<tokens.length; i++)
        {
            if (tokens[i] == null ? "" != null : ! tokens[i].equals(""))
            {
                t[ndx] = tokens[i];
                ndx++;
            }
        }

        shape.colorCode = new Integer(t[1]);
        
        if (BFCFlag)
        {
            shape.p1 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            shape.p2 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));
            shape.p3 = new Point3f(new Float(t[8]), new Float(t[9]), new Float(t[10]));
        }
        else
        {
            shape.p3 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            shape.p2 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));
            shape.p1 = new Point3f(new Float(t[8]), new Float(t[9]), new Float(t[10]));
        }

        // apply orientation from parent
        shape.p1 = orientation.times(shape.p1);
        shape.p2 = orientation.times(shape.p2);
        shape.p3 = orientation.times(shape.p3);

        // this must be counter clockwise
        Point3f edge1 = shape.p2.minus(shape.p1);
        Point3f edge2 = shape.p3.minus(shape.p2);
        shape.norm = edge2.crossProd(edge1).unitVector();

        return shape;
    }

    @Override
    public void setOrientation(Matrix4f orientation)
    {
        p1 = orientation.times(p1);
        p2 = orientation.times(p2);
        p3 = orientation.times(p3);
    }

    @Override
    public void addToGL(GL gl)
    {
        gl.glBegin(GL.GL_TRIANGLES);
        gl.glColor4f(color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, color.getAlpha() / 255f);
        gl.glVertex3f(p1.x, p1.y, p1.z);
        gl.glVertex3f(p2.x, p2.y, p2.z);
        gl.glVertex3f(p3.x, p3.y, p3.z);
        gl.glEnd();
    }

    @Override
    public void addToGLwithBFC(GL gl)
    {
        gl.glBegin(GL.GL_TRIANGLES);
        gl.glColor4f(color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, color.getAlpha() / 255f);
        gl.glNormal3f(norm.x, norm.y, norm.z);
        gl.glVertex3f(p1.x, p1.y, p1.z);
        gl.glVertex3f(p2.x, p2.y, p2.z);
        gl.glVertex3f(p3.x, p3.y, p3.z);
        gl.glEnd();
    }

    @Override
    public void addToList(Map<MaterialTypes, ArrayList<Shape>> materials, int requestedColorCode, float lineWidth)
    {
        setColor(requestedColorCode, this.colorCode);

        for (MaterialTypes materialType : material.materialTypes)
            materials.get(materialType).add(this);
    }

    @Override
    public void addToListBFC(Map<MaterialTypes, ArrayList<Shape>> materials, int requestedColorCode, float lineWidth, boolean BFCflag)
    {
        setColor(requestedColorCode, this.colorCode);

        if (!BFCflag)
        {
            Point3f temp = p1.copy();
            p1 = p3;
            p3 = temp;
        }

        for (MaterialTypes materialType : material.materialTypes)
            materials.get(materialType).add(this);
    }

    @Override
    public Point3f getCentroid (Point3f min, Point3f max)
    {
        minPoint = new Point3f(Float.MAX_VALUE, Float.MAX_VALUE, Float.MAX_VALUE);
        maxPoint = new Point3f(-Float.MAX_VALUE, -Float.MAX_VALUE, -Float.MAX_VALUE);

        if (p1.x < minPoint.x)
            minPoint.x = p1.x;
        if (p1.x > maxPoint.x)
            maxPoint.x = p1.x;
        if (p1.y < minPoint.y)
            minPoint.y = p1.y;
        if (p1.y > maxPoint.y)
            maxPoint.y = p1.y;
        if (p1.z < minPoint.z)
            minPoint.z = p1.z;
        if (p1.z > maxPoint.z)
            maxPoint.z = p1.z;

        if (p2.x < minPoint.x)
            minPoint.x = p2.x;
        if (p2.x > maxPoint.x)
            maxPoint.x = p2.x;
        if (p2.y < minPoint.y)
            minPoint.y = p2.y;
        if (p2.y > maxPoint.y)
            maxPoint.y = p2.y;
        if (p2.z < minPoint.z)
            minPoint.z = p2.z;
        if (p2.z > maxPoint.z)
            maxPoint.z = p2.z;

        if (p3.x < minPoint.x)
            minPoint.x = p3.x;
        if (p3.x > maxPoint.x)
            maxPoint.x = p3.x;
        if (p3.y < minPoint.y)
            minPoint.y = p3.y;
        if (p3.y > maxPoint.y)
            maxPoint.y = p3.y;
        if (p3.z < minPoint.z)
            minPoint.z = p3.z;
        if (p3.z > maxPoint.z)
            maxPoint.z = p3.z;

        if (minPoint.x < min.x)
            min.x = minPoint.x;
        if (maxPoint.x > max.x)
            max.x = maxPoint.x;
        if (minPoint.y < min.y)
            min.y = minPoint.y;
        if (maxPoint.y > max.y)
            max.y = maxPoint.y;
        if (minPoint.z < min.z)
            min.z = minPoint.z;
        if (maxPoint.z > max.z)
            max.z = maxPoint.z;

        centroid = new Point3f(maxPoint.x - (maxPoint.x - minPoint.x) / 2f, maxPoint.y - (maxPoint.y - minPoint.y) / 2f, maxPoint.z - (maxPoint.z - minPoint.z) / 2f);

        return centroid;
    }

    @Override
    public boolean contains (float x, float y, ViewAngleModes viewAngle)
    {
        int crossings = 0;

        if(viewAngle == ViewAngleModes.FRONT || viewAngle == ViewAngleModes.BACK)
        {
            if (((p1.y <= y) && (y < p2.y) || (p2.y <= y) && (y < p1.y)) && x < ((p2.x - p1.x) / (p2.y - p1.y)) * (y - p1.y) + p1.x)
                crossings++;
            if (((p2.y <= y) && (y < p3.y) || (p3.y <= y) && (y < p2.y)) && x < ((p3.x - p2.x) / (p3.y - p2.y)) * (y - p2.y) + p2.x)
                crossings++;
            if (((p3.y <= y) && (y < p1.y) || (p1.y <= y) && (y < p3.y)) && x < ((p1.x - p3.x) / (p1.y - p3.y)) * (y - p3.y) + p3.x)
                crossings++;
        }
        else if(viewAngle == ViewAngleModes.LEFT || viewAngle == ViewAngleModes.RIGHT)
        {
            if (((p1.y <= y) && (y < p2.y) || (p2.y <= y) && (y < p1.y)) && x < ((p2.z - p1.z) / (p2.y - p1.y)) * (y - p1.y) + p1.z)
                crossings++;
            if (((p2.y <= y) && (y < p3.y) || (p3.y <= y) && (y < p2.y)) && x < ((p3.z - p2.z) / (p3.y - p2.y)) * (y - p2.y) + p2.z)
                crossings++;
            if (((p3.y <= y) && (y < p1.y) || (p1.y <= y) && (y < p3.y)) && x < ((p1.z - p3.z) / (p1.y - p3.y)) * (y - p3.y) + p3.z)
                crossings++;
        }
        else if(viewAngle == ViewAngleModes.TOP || viewAngle == ViewAngleModes.BOTTOM)
        {
            if (((p1.z <= y) && (y < p2.z) || (p2.z <= y) && (y < p1.z)) && x < ((p2.x - p1.x) / (p2.z - p1.z)) * (y - p1.z) + p1.x)
                crossings++;
            if (((p2.z <= y) && (y < p3.z) || (p3.z <= y) && (y < p2.z)) && x < ((p3.x - p2.x) / (p3.z - p2.z)) * (y - p2.z) + p2.x)
                crossings++;
            if (((p3.z <= y) && (y < p1.z) || (p1.z <= y) && (y < p3.z)) && x < ((p1.x - p3.x) / (p1.z - p3.z)) * (y - p3.z) + p3.x)
                crossings++;
        }

        if (crossings % 2 != 0)
        {
            return true;
        }
        return false;
    }

    @Override
    public Triangle copy()
    {
        Triangle that = new Triangle();

        that.color = this.color;
        that.colorCode = this.colorCode;
        if (this.material != null)
            that.material = this.material.copy();
        that.centroid = this.centroid.copy();
        that.minPoint = this.minPoint.copy();
        that.maxPoint = this.maxPoint.copy();

        that.norm = this.norm.copy();
        that.p1 = this.p1.copy();
        that.p2 = this.p2.copy();
        that.p3 = this.p3.copy();

        return that;
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append(super.toString()).append("\n");
        strBuffer.append("p1 ").append(p1).append("\n");
        strBuffer.append("p2 ").append(p2).append("\n");
        strBuffer.append("p3 ").append(p3).append("\n");

        return strBuffer.toString();
    }
}