
package com.locad.main;

import com.locad.enumerations.Enumerations.ViewAngleModes;
import com.locad.ldraw.Part;
import com.locad.ldraw.PartsLibrary;
import com.locad.viewport.Canvas;
import com.locad.viewport.Scene;
import com.locad.viewport.WBorder;
import java.awt.*;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.WindowConstants;
import org.apache.log4j.Logger;

public class Desktop extends JFrame
{
    private final Logger log;

    private final Scene scene;

    public Canvas front;
    public Canvas right;
    public Canvas top;
    public Canvas isometric;
    public List<Canvas> partsCanvases = new ArrayList<Canvas>();

    private JPanel partsPanel;
    private JPanel viewPanel;

    public Desktop (Part part, int defaultColorCode, Color bgColor, float lineWidth, boolean enableBFC)
    {
        log = Logger.getLogger(getClass().getName());

        this.scene = new Scene(part, lineWidth, enableBFC);

        createPartsPanel(defaultColorCode, bgColor, lineWidth);
        createViewPanel(bgColor);

        init();

        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        setLayout(layout);

        c.fill = GridBagConstraints.BOTH;

        c.weightx = 0.10;
        c.weighty = 1.0;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(0, 0, 0, 0);
        layout.setConstraints(partsPanel, c);
        add(partsPanel);

        c.weightx = 0.90;
        c.weighty = 1.0;
        c.gridx = 1;
        c.gridy = 0;
        c.insets = new Insets(0, 0, 0, 0);
        layout.setConstraints(viewPanel, c);
        add(viewPanel);

//        front.requestFocus();
//        right.requestFocus();
//        top.requestFocus();
//        isometric.requestFocus();
//        for (Canvas partCanvas : partsCanvases)
//        {
//            partCanvas.requestFocus();
//        }
    }

    private void init()
    {
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        this.setPreferredSize(new Dimension(1600, 1024));
        this.setSize(new Dimension(1600, 1024));

        if (scene.getPart() != null)
            setTitle(scene.getPart().getFileName());

//        GroupLayout panelLayout = new GroupLayout(panel);
//        panel.setLayout(panelLayout);
//        panelLayout.setHorizontalGroup(panelLayout.createParallelGroup(Alignment.LEADING).addGap(0, 380, Short.MAX_VALUE));
//        panelLayout.setVerticalGroup(panelLayout.createParallelGroup(Alignment.LEADING).addGap(0, 258, Short.MAX_VALUE));
//
//        GroupLayout layout = new GroupLayout(getContentPane());
//        getContentPane().setLayout(layout);
//        layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING).addGroup(layout.createSequentialGroup().addContainerGap().addGroup(layout.createParallelGroup(Alignment.LEADING).addComponent(panel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addComponent(label)).addContainerGap()));
//        layout.setVerticalGroup(layout.createParallelGroup(Alignment.LEADING).addGroup(layout.createSequentialGroup().addContainerGap().addComponent(label).addPreferredGap(ComponentPlacement.RELATED).addComponent(panel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addContainerGap()));
    }

    private void createPartsPanel(int colorCode, Color bgColor, float lineWidth)
    {
        partsPanel = new JPanel();
        partsPanel.setBackground(bgColor);
        partsPanel.setBorder(new WBorder());
        partsPanel.setLayout(new GridLayout(14, 3));

        List<String> partNames = PartsLibrary.getInstance().getPartNamesFromPath("C:/LDRAW/P/");

//        List<String> partNames = new ArrayList<String>();
//        for (String fileName : PartsLibrary.getInstance().partDescriptions.keySet())
//        {
//            if (PartsLibrary.getInstance().partDescriptions.get(fileName).startsWith("B") || PartsLibrary.getInstance().partDescriptions.get(fileName).startsWith("b"))
//            {
//                partNames.add(fileName);
//            }
//        }

        for (int i=0; i<Math.min(42, partNames.size()); i++)
        {
            Part part = PartsLibrary.getInstance().getPart(partNames.get(i));
            part.colorCode = colorCode;
            Canvas partBag = new Canvas(ViewAngleModes.PARTS_BAG, new Scene(part, lineWidth, false), bgColor);

            partsPanel.add(partBag);
            partsCanvases.add(partBag);
        }
    }

    private void createViewPanel(Color bgColor)
    {
        front = new Canvas(ViewAngleModes.FRONT, scene, bgColor);
        right = new Canvas(ViewAngleModes.RIGHT, scene, bgColor);
        top = new Canvas(ViewAngleModes.TOP, scene, bgColor);
        isometric = new Canvas(ViewAngleModes.ISOMETRIC, scene, bgColor);

        JPanel frontPanel = new JPanel();
        JPanel rightPanel = new JPanel();
        JPanel topPanel = new JPanel();
        JPanel isoPanel = new JPanel();

        frontPanel.setLayout(new BorderLayout());
        rightPanel.setLayout(new BorderLayout());
        topPanel.setLayout(new BorderLayout());
        isoPanel.setLayout(new BorderLayout());

        frontPanel.setBorder(new WBorder());
        rightPanel.setBorder(new WBorder());
        topPanel.setBorder(new WBorder());
        isoPanel.setBorder(new WBorder());

        frontPanel.setBackground(bgColor);
        rightPanel.setBackground(bgColor);
        topPanel.setBackground(bgColor);
        isoPanel.setBackground(bgColor);

        frontPanel.add(front, BorderLayout.CENTER);
        rightPanel.add(right, BorderLayout.CENTER);
        topPanel.add(top, BorderLayout.CENTER);
        isoPanel.add(isometric, BorderLayout.CENTER);

        viewPanel = new JPanel();
        viewPanel.setBackground(bgColor);
        viewPanel.setLayout(new GridLayout(2,2));
        viewPanel.add(frontPanel);
        viewPanel.add(rightPanel);
        viewPanel.add(topPanel);
        viewPanel.add(isoPanel);
    }
}
