
package com.locad.main;

import com.locad.ldraw.ColorLibrary;
import com.locad.ldraw.Part;
import com.locad.ldraw.PartsLibrary;
import java.awt.Color;
import java.awt.EventQueue;
import java.util.ArrayList;
import java.util.List;
import javax.swing.UIManager;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

public class Main
{
    private static final Logger log = Logger.getLogger(Main.class);

    private static final boolean keepLogs = false;

    private static final float lineWidth = 2f;
    private static final int defaultColorCode = 4;
    private static final int defaultPartsBagColorCode = 0;
    private static final Color bgColor = Color.WHITE;
    private static final String ldrawDir = "C:/LDraw/";
    private static final String partListFile = ldrawDir + "Primitives.lst";
    private static final String colorFile = ldrawDir + "LDConfig.ldr";
    private static final boolean enableBFC = true;

    private static final String[] primPath0 =  { ldrawDir + "P/", "" };
    private static final String[] primPath1 =  { ldrawDir + "P/", "48/" };
    private static final String[] partPath0 =  { ldrawDir + "PARTS/", "" };
    private static final String[] partPath1 =  { ldrawDir + "PARTS/", "S/" };

    private static final String partFile = ldrawDir + "parts/2546p01.DAT";
    //private static final String partFile = ldrawDir + "parts/3001.DAT";
    //private static final String partFile = ldrawDir + "parts/S/6024S3.DAT";
    //private static final String partFile = ldrawDir + "parts/S/3001s01.DAT";
    //private static final String partFile = ldrawDir + "parts/6581.DAT";

    public Main()
    {
        // create color library
        ColorLibrary.getInstance().readLDConfig(colorFile);

        List<String[]> primitiveDirs = new ArrayList<String[]>();
        List<String[]> partDirs = new ArrayList<String[]>();
        
        primitiveDirs.add(primPath0);
        primitiveDirs.add(primPath1);
        partDirs.add(partPath0);
        partDirs.add(partPath1);

        // create parts library
        PartsLibrary.getInstance().readPartFiles(primitiveDirs, partDirs, ldrawDir, partListFile);

        // create part for editing
        Part part = Part.from_File(partFile);
        part.colorCode = defaultColorCode;

        Desktop desktop = new Desktop(part, defaultPartsBagColorCode, bgColor, lineWidth, enableBFC);
        desktop.setVisible(true);
    }

    public static void main(String[] args)
    {
        if (keepLogs)
            LoggingUtils.setupLog4j("LOCAD.log");
        else
            DOMConfigurator.configureAndWatch("config/logger.xml", 500);
        log.info("LEGO Open CAD started");

        // Run this in the AWT event thread to prevent deadlocks and race conditions
        EventQueue.invokeLater(new Runnable()
        {
            @Override
            public void run()
            {
                try
                {
                    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                }
                catch (Exception ex)
                {
                    log.info("Can not enable system look and feel " + ex);
                }

                Main m = new Main();
            }
        });
    }
}
