
package com.locad.viewport;

import com.locad.enumerations.Enumerations.ViewAngleModes;
import com.locad.geometry.Shape;
import com.locad.geometry.Vec4f;
import com.locad.ldraw.Part;
import java.awt.Color;
import java.awt.Point;
import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;
import org.apache.log4j.Logger;

public class Renderer implements GLEventListener
{
    private final Logger log;

    public ViewAngleModes viewAngle;

    private GLU glu = new GLU();
    private boolean reCreateScene = false;

    private final Scene scene;
    private Camera camera;
    
    private Point rotationPoint = new Point();

    private float aspect = 0;
    private float width = 0;
    private float height = 0;

    private final Color bgColor;

    public Renderer (ViewAngleModes viewAngle, Scene scene, Color bgColor)
    {
        log = Logger.getLogger(getClass().getName());

        this.viewAngle = viewAngle;
        this.scene = scene;
        this.bgColor = bgColor;

        this.camera = new Camera(viewAngle);
    }

    public void init(GLAutoDrawable glDrawable)
    {
        final GL gl = glDrawable.getGL();

        gl.glShadeModel (GL.GL_SMOOTH);

        gl.glClearColor(bgColor.getRed() / 255f, bgColor.getGreen() / 255f, bgColor.getBlue() / 255, bgColor.getAlpha() / 255f);
        gl.glClearDepth(1.0f);

        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDepthFunc(GL.GL_LEQUAL);

        gl.glPolygonMode(GL.GL_FRONT, GL.GL_FILL);

        if (scene.getEnableBFC())
        {
//            gl.glShadeModel(mode);
//            gl.glShaderSource(shader, count, string, null);

//            gl.glCullFace(GL.GL_BACK);
//            gl.glEnable(GL.GL_CULL_FACE);

            Light light0 = new Light(new float[] {0.0f, 0.0f, 0.0f, 1.0f}, new float[] {0.92f, 0.92f, 0.92f, 1.0f}, new float[] {0.2f, 0.2f, 0.2f, 1.0f}, new Vec4f(camera.pos.x, camera.pos.y, camera.pos.z, -1f));
            light0.ApplyToGL(gl, GL.GL_LIGHT0);
        }
        else
        {
            gl.glDisable(GL.GL_LIGHTING);
            gl.glDisable(GL.GL_BLEND);
        }

        scene.createScene(gl);

        Listener listener = new Listener(this, camera);
        //glDrawable.addKeyListener(listener);
        glDrawable.addMouseListener(listener);
        glDrawable.addMouseMotionListener(listener);
        glDrawable.addMouseWheelListener(listener);
    }

    public void display(GLAutoDrawable glDrawable)
    {
        final GL gl = glDrawable.getGL();

        //if (reCreateScene)
        {
            scene.createScene(gl);
            reCreateScene = false;
        }

        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        gl.glOrtho(camera.left, camera.right, camera.bottom, camera.top, camera.near, camera.far);
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();

        if (viewAngle == ViewAngleModes.PARTS_BAG)
        {
            camera.startDrag(rotationPoint);
            rotationPoint.x += 1;
            rotationPoint.y += 1;
            camera.rotate(rotationPoint);
        }

        glu.gluLookAt(camera.pos.x, camera.pos.y, camera.pos.z, camera.lookAt.x, camera.lookAt.y, camera.lookAt.z, camera.yaw, camera.roll, camera.pitch);
        
//        Point3f pos = camera.pos.copy();
//        pos.x = -pos.x;
//        pos.y = -pos.y;
//        pos.z = -pos.z;
//        gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, camera.pos.toArray(), 0);

        // Render scene by calling display list
        gl.glCallList(scene.getScene_dl());
        gl.glFlush();
    }

    public void doSelection(Point p, boolean mouseDown)
    {
        float x;
        float y;
        Shape minShape = null;

        // translate and normalize screen point to scene point
        if (viewAngle == ViewAngleModes.FRONT)
        {
            x = p.x / width * (camera.right - camera.left) + camera.left + camera.lookAt.x;
            y = p.y / height * (camera.top - camera.bottom) + camera.bottom + camera.lookAt.y;

            float value = Float.MAX_VALUE;
            for (Shape shape : scene.getPart().getShapes())
            {
                if (shape instanceof Part)
                {
                    if (shape.contains(x, y, viewAngle) && shape.minPoint.z < value)
                    {
                        value = shape.minPoint.z;
                        minShape = shape;
                    }
                }
            }
        }
        else if(viewAngle == ViewAngleModes.BACK)
        {
            x = (width - p.x) / width * (camera.right - camera.left) + camera.left + camera.lookAt.x;
            y = p.y / height * (camera.top - camera.bottom) + camera.bottom + camera.lookAt.y;

            float value = -Float.MAX_VALUE;
            for (Shape shape : scene.getPart().getShapes())
            {
                if (shape instanceof Part)
                {
                    if (shape.contains(x, y, viewAngle) && shape.maxPoint.z > value)
                    {
                        value = shape.maxPoint.z;
                        minShape = shape;
                    }
                }
            }
        }
        else if (viewAngle == ViewAngleModes.RIGHT)
        {
            x = p.x / width * (camera.right - camera.left) + camera.left + camera.lookAt.z;
            y = p.y / height * (camera.top - camera.bottom) + camera.bottom + camera.lookAt.y;

            float value = -Float.MAX_VALUE;
            for (Shape shape : scene.getPart().getShapes())
            {
                if (shape instanceof Part)
                {
                    if (shape.contains(x, y, viewAngle) && shape.maxPoint.x > value)
                    {
                        value = shape.maxPoint.x;
                        minShape = shape;
                    }
                }
            }
        }
        else if (viewAngle == ViewAngleModes.LEFT)
        {
            x = (width -p.x) / width * (camera.right - camera.left) + camera.left + camera.lookAt.z;
            y = p.y / height * (camera.top - camera.bottom) + camera.bottom + camera.lookAt.y;

            float value = Float.MAX_VALUE;
            for (Shape shape : scene.getPart().getShapes())
            {
                if (shape instanceof Part)
                {
                    if (shape.contains(x, y, viewAngle) && shape.minPoint.x < value)
                    {
                        value = shape.minPoint.x;
                        minShape = shape;
                    }
                }
            }
        }
        else if(viewAngle == ViewAngleModes.TOP)
        {
            x = p.x / width * (camera.right - camera.left) + camera.left + camera.lookAt.x;
            y = (height - p.y) / height * (camera.top - camera.bottom) + camera.bottom + camera.lookAt.z;

            float value = Float.MAX_VALUE;
            for (Shape shape : scene.getPart().getShapes())
            {
                if (shape instanceof Part)
                {
                    if (shape.contains(x, y, viewAngle) && shape.minPoint.y < value)
                    {
                        value = shape.minPoint.y;
                        minShape = shape;
                    }
                }
            }
        }
        else if(viewAngle == ViewAngleModes.BOTTOM)
        {
            x = (width - p.x) / width * (camera.right - camera.left) + camera.left + camera.lookAt.x;
            y = (height - p.y) / height * (camera.top - camera.bottom) + camera.bottom + camera.lookAt.z;

            float value = -Float.MAX_VALUE;
            for (Shape shape : scene.getPart().getShapes())
            {
                if (shape instanceof Part)
                {
                    if (shape.contains(x, y, viewAngle) && shape.maxPoint.y > value)
                    {
                        value = shape.maxPoint.y;
                        minShape = shape;
                    }
                }
            }
        }

        if (minShape != null)
        {
            if (minShape instanceof Part)
            {
                log.info(((Part) minShape).colorCode);
                minShape.setColor(0, 16);
                reCreateScene = true;
            }
        }
    }

    public void displayChanged(GLAutoDrawable glDrawable, boolean modeChanged, boolean deviceChanged)
    {
        //init(glDrawable);
    }

    public void reshape(GLAutoDrawable glDrawable, int x, int y, int w, int h)
    {
        camera.calcViewBox(scene.getCentroid(), scene.getMinPoint(), scene.getMaxPoint());

        aspect = (float) w / (float) h;
        width = w;
        height = h;
        if ( aspect < 1.0 )
        { // window taller than wide
            camera.bottom /= aspect;
            camera.top /= aspect;
        }
        else
        {
            camera.left *= aspect;
            camera.right *= aspect;
        }

        final GL gl = glDrawable.getGL();
        gl.glViewport(0, 0, w, h);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        gl.glOrtho(camera.left, camera.right, camera.bottom, camera.top, camera.near, camera.far);
        camera.setBounds((float) w, (float) h);
    }
}
