
package com.locad.viewport;

import java.awt.*;
import javax.swing.border.AbstractBorder;

public class WBorder extends AbstractBorder
{
    private int borderArcWidth = 8;
    private int borderArcHeight = 8;
    private int leftInset = 4;
    private int topInset = 4;
    private int rightInset = 4;
    private int bottomInset = 4;
    private Color borderColor = new Color(0, 0, 0);

    public WBorder()
    {
    }

    public WBorder(int inset)
    {
        this.leftInset = this.topInset = this.rightInset = this.bottomInset = inset;
    }

    public WBorder(int topInset, int leftInset, int bottomInset, int rightInset)
    {
        this.leftInset = leftInset;
        this.topInset = topInset;
        this.rightInset = rightInset;
        this.bottomInset = bottomInset;
    }

    @Override
    public Insets getBorderInsets(Component c)
    {
        return new Insets(topInset, leftInset, bottomInset, rightInset);
    }

    @Override
    public Insets getBorderInsets(Component c, Insets insets)
    {
        insets.left = this.leftInset;
        insets.top = this.topInset;
        insets.right = this.rightInset;
        insets.bottom = this.bottomInset;
        return insets;
    }

    @Override
    public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
    {
        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        g2.setColor(borderColor);

        g2.drawArc(x, y, borderArcWidth, borderArcHeight, 90, 90); // Top Left
        // Arc
        g2.drawLine(x + borderArcWidth / 2, y, x + width - 1 - borderArcWidth / 2, y); // Top
        // Line
        g2.drawArc(x + width - 1 - borderArcWidth, y, borderArcWidth, borderArcHeight, 0, 90); // Top
        // Right
        // Arc
        g2.drawLine(x + width - 1, y + borderArcHeight / 2, x + width - 1, y + height - 2 - borderArcHeight / 2); // Right
        // Line
        g2.drawArc(x + width - 1 - borderArcWidth, y + height - 2 - borderArcHeight, borderArcWidth, borderArcHeight, 270, 90); // Bottom
        // Right
        // Arc
        g2.drawLine(x + width - 1 - borderArcWidth / 2, y + height - 2, x + borderArcWidth / 2, y + height - 2); // Bottom
        // Line
        g2.drawArc(x, y + height - 2 - borderArcHeight, borderArcHeight, borderArcWidth, 180, 90); // Bottom
        // Left
        // Arc
        g2.drawLine(x, y + height - 2 - borderArcHeight / 2, x, y + borderArcHeight / 2); // Left
        // Line
    }

    public Color getBorderColor()
    {
        return borderColor;
    }

    public void setBorderColor(Color borderColor)
    {
        this.borderColor = borderColor;
    }
}
