
package com.locad.geometry;

import com.locad.enumerations.Enumerations.MaterialTypes;
import com.locad.enumerations.Enumerations.ViewAngleModes;
import com.locad.model.Color4b;
import javax.media.opengl.GL;

public class Line extends Shape
{
    public final Point3f p1;
    public final Point3f p2;

    public Line(Point3f p1, Point3f p2, short colorCode)
    {
        this.p1 = p1;
        this.p2 = p2;
        this.colorCode = colorCode;
    }

    public static Line from_Line (String input, boolean BFCFlag)
    {
        String[] tokens = input.trim().split(" ");
        String[] t = new String[tokens.length];
        int ndx = 0;
        for (int i=0; i<tokens.length; i++)
        {
            if (tokens[i] == null ? "" != null : ! tokens[i].equals(""))
            {
                t[ndx] = tokens[i];
                ndx++;
            }
        }

        if (BFCFlag)
        {
            Point3f p1 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            Point3f p2 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));

            return new Line(p1, p2, new Short(t[1]));
        }
        else
        {
            Point3f p2 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            Point3f p1 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));

            return new Line(p1, p2, new Short(t[1]));
        }
    }
    
    @Override
    public void addToGL(GL gl, short requestedColorCode, MaterialTypes materialType)
    {
        Color4b color = getColor(requestedColorCode, materialType);
        if (color != null)
        {
            gl.glBegin(GL.GL_LINES);
            gl.glColor4f((color.red + 128) / 255f, (color.green + 128) / 255f, (color.blue + 128) / 255f, (color.alpha + 128) / 255f);
            gl.glVertex3f(p1.x, p1.y, p1.z);
            gl.glVertex3f(p2.x, p2.y, p2.z);
            gl.glEnd();
        }
    }

    @Override
    public void addToGLwithBFC(GL gl, short requestedColorCode, MaterialTypes materialType, boolean BFCflag)
    {
        Color4b color = getColor(requestedColorCode, materialType);
        if (color != null)
        {
            Point3f temp1;
            Point3f temp2;

            if (BFCflag)
            {
                temp1 = (p2);
                temp2 = (p1);
            }
            else
            {
                temp1 = (p1);
                temp2 = (p2);
            }

//            // compute the norm - this must be counter clockwise
//            Point3f edge1 = temp2.minus(temp1);
//            Point3f edge2 = temp3.minus(temp1);
//            Point3f norm = edge1.crossProd(edge2).unitVector();

            gl.glBegin(GL.GL_LINES);
            gl.glColor4f((color.red + 128) / 255f, (color.green + 128) / 255f, (color.blue + 128) / 255f, (color.alpha + 128) / 255f);
            //gl.glNormal3f(p1.x + norm.x, p1.y + norm.y, p1.z + norm.z);
            gl.glNormal3f(-1f, -1f, -1f);
            gl.glVertex3f(temp1.x, temp1.y, temp1.z);
            gl.glVertex3f(temp2.x, temp2.y, temp2.z);
            gl.glEnd();
        }
    }

    @Override
    public void getBoundingBox (Matrix4f orientation, Point3f min, Point3f max)
    {
        Point3f temp1 = orientation.times(p1);
        Point3f temp2 = orientation.times(p2);

        if (temp1.x < min.x)
            min.x = temp1.x;
        if (temp1.x > max.x)
            max.x = temp1.x;
        if (temp1.y < min.y)
            min.y = temp1.y;
        if (temp1.y > max.y)
            max.y = temp1.y;
        if (temp1.z < min.z)
            min.z = temp1.z;
        if (temp1.z > max.z)
            max.z = temp1.z;

        if (temp2.x < min.x)
            min.x = temp2.x;
        if (temp2.x > max.x)
            max.x = temp2.x;
        if (temp2.y < min.y)
            min.y = temp2.y;
        if (temp2.y > max.y)
            max.y = temp2.y;
        if (temp2.z < min.z)
            min.z = temp2.z;
        if (temp2.z > max.z)
            max.z = temp2.z;
    }

    @Override
    public boolean contains (Matrix4f orientation, Point3f line, ViewAngleModes viewAngle)
    {
        return false;

//        float t = 0.2f;
//        Point3f temp1 = orientation.times(new Point3f(p1.x - t, p1.y - t, p1.z - t));
//        Point3f temp2 = orientation.times(new Point3f(p2.x - t, p2.y - t, p2.z - t));
//        Point3f temp3 = orientation.times(new Point3f(p2.x + t, p2.y + t, p2.z + t));
//        Point3f temp4 = orientation.times(new Point3f(p1.x + t, p1.y + t, p1.z + t));
//
//        int crossings = 0;
//
//        if(viewAngle == ViewAngleModes.FRONT || viewAngle == ViewAngleModes.BACK)
//        {
//            if (((temp1.y <= line.y) && (line.y < temp2.y) || (temp2.y <= line.y) && (line.y < temp1.y)) && line.x < ((temp2.x - temp1.x) / (temp2.y - temp1.y)) * (line.y - temp1.y) + temp1.x)
//                crossings++;
//            if (((temp2.y <= line.y) && (line.y < temp3.y) || (temp3.y <= line.y) && (line.y < temp2.y)) && line.x < ((temp3.x - temp2.x) / (temp3.y - temp2.y)) * (line.y - temp2.y) + temp2.x)
//                crossings++;
//            if (((temp3.y <= line.y) && (line.y < temp4.y) || (temp4.y <= line.y) && (line.y < temp3.y)) && line.x < ((temp4.x - temp3.x) / (temp4.y - temp3.y)) * (line.y - temp3.y) + temp3.x)
//                crossings++;
//            if (((temp4.y <= line.y) && (line.y < temp1.y) || (temp1.y <= line.y) && (line.y < temp4.y)) && line.x < ((temp1.x - temp4.x) / (temp1.y - temp4.y)) * (line.y - temp4.y) + temp4.x)
//                crossings++;
//        }
//        else if(viewAngle == ViewAngleModes.LEFT || viewAngle == ViewAngleModes.RIGHT)
//        {
//            if (((temp1.y <= line.y) && (line.y < temp2.y) || (temp2.y <= line.y) && (line.y < temp1.y)) && line.z < ((temp2.z - temp1.z) / (temp2.y - temp1.y)) * (line.y - temp1.y) + temp1.z)
//                crossings++;
//            if (((temp2.y <= line.y) && (line.y < temp3.y) || (temp3.y <= line.y) && (line.y < temp2.y)) && line.z < ((temp3.z - temp2.z) / (temp3.y - temp2.y)) * (line.y - temp2.y) + temp2.z)
//                crossings++;
//            if (((temp3.y <= line.y) && (line.y < temp4.y) || (temp4.y <= line.y) && (line.y < temp3.y)) && line.z < ((temp4.z - temp3.z) / (temp4.y - temp3.y)) * (line.y - temp3.y) + temp3.z)
//                crossings++;
//            if (((temp4.y <= line.y) && (line.y < temp1.y) || (temp1.y <= line.y) && (line.y < temp4.y)) && line.z < ((temp1.z - temp4.z) / (temp1.y - temp4.y)) * (line.y - temp4.y) + temp4.z)
//                crossings++;
//        }
//        else if(viewAngle == ViewAngleModes.TOP || viewAngle == ViewAngleModes.BOTTOM)
//        {
//            if (((temp1.z <= line.z) && (line.z < temp2.z) || (temp2.z <= line.z) && (line.z < temp1.z)) && line.x < ((temp2.x - temp1.x) / (temp2.z - temp1.z)) * (line.z - temp1.z) + temp1.x)
//                crossings++;
//            if (((temp2.z <= line.z) && (line.z < temp3.z) || (temp3.z <= line.z) && (line.z < temp2.z)) && line.x < ((temp3.x - temp2.x) / (temp3.z - temp2.z)) * (line.z - temp2.z) + temp2.x)
//                crossings++;
//            if (((temp3.z <= line.z) && (line.z < temp4.z) || (temp4.z <= line.z) && (line.z < temp3.z)) && line.x < ((temp4.x - temp3.x) / (temp4.z - temp3.z)) * (line.z - temp3.z) + temp3.x)
//                crossings++;
//            if (((temp4.z <= line.z) && (line.z < temp1.z) || (temp1.z <= line.z) && (line.z < temp4.z)) && line.x < ((temp1.x - temp4.x) / (temp1.z - temp4.z)) * (line.z - temp4.z) + temp4.x)
//                crossings++;
//        }
//
//        return crossings % 2 != 0;
    }

    @Override
    public Line copy()
    {
        return new Line(this.p1, this.p2, this.colorCode);
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append("shapeType ").append("Line").append("\n");
        strBuffer.append(super.toString());
        strBuffer.append("p1 ").append(p1).append("\n");
        strBuffer.append("p2 ").append(p2).append("\n");

        return strBuffer.toString();
    }

    @Override
    public String toString (Matrix4f orientation)
    {
        Point3f temp1 = orientation.times(p1);
        Point3f temp2 = orientation.times(p2);

        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append("shapeType ").append("Line").append("\n");
        strBuffer.append(super.toString());
        strBuffer.append("p1 ").append(temp1).append("\n");
        strBuffer.append("p2 ").append(temp2).append("\n");

        return strBuffer.toString();
    }
}