
package com.locad.geometry;

public class Matrix4f
{
    //  0  1  2  3
    //  4  5  6  7
    //  8  9 10 11
    // 12 13 14 15

    private final float[] matrix;

    public Matrix4f(float[] matrix)
    {
        this.matrix = matrix;
    }

    public Matrix4f(float a0, float b0, float c0, float d0,
                    float a1, float b1, float c1, float d1,
                    float a2, float b2, float c2, float d2,
                    float a3, float b3, float c3, float d3)
    {
        this.matrix = new float[16];

        this.matrix[0]  = a0;
        this.matrix[4]  = a1;
        this.matrix[8]  = a2;
        this.matrix[12] = a3;
        this.matrix[1]  = b0;
        this.matrix[5]  = b1;
        this.matrix[9]  = b2;
        this.matrix[13] = b3;
        this.matrix[2]  = c0;
        this.matrix[6]  = c1;
        this.matrix[10] = c2;
        this.matrix[14] = c3;
        this.matrix[3]  = d0;
        this.matrix[7]  = d1;
        this.matrix[11] = d2;
        this.matrix[15] = d3;
    }

    public Matrix4f(Vec4f q1)
    {
        this.matrix = new float[16];
        
        float n = (q1.x * q1.x) + (q1.y * q1.y) + (q1.z * q1.z) + (q1.w * q1.w);
        float s = (n > 0.0f) ? (2.0f / n) : 0.0f;

        float xs = q1.x * s;
        float ys = q1.y * s;
        float zs = q1.z * s;
        float wx = q1.w * xs;
        float wy = q1.w * ys;
        float wz = q1.w * zs;
        float xx = q1.x * xs;
        float xy = q1.x * ys;
        float xz = q1.x * zs;
        float yy = q1.y * ys;
        float yz = q1.y * zs;
        float zz = q1.z * zs;

        this.matrix[0]  = 1.0f - (yy + zz);
        this.matrix[1]  = xy - wz;
        this.matrix[2]  = xz + wy;
        this.matrix[3]  = 0f;
        this.matrix[4]  = xy + wz;
        this.matrix[5]  = 1.0f - (xx + zz);
        this.matrix[6]  = yz - wx;
        this.matrix[7]  = 0f;
        this.matrix[8]  = xz - wy;
        this.matrix[9]  = yz + wx;
        this.matrix[10] = 1.0f - (xx + yy);
        this.matrix[11] = 0f;
        this.matrix[12] = 0;
        this.matrix[13] = 0;
        this.matrix[14] = 0;
        this.matrix[15] = 1f;
    } 

    public static Matrix4f zeroMatrix ()
    {
        return new Matrix4f(0f, 0f, 0f, 0f,
                            0f, 0f, 0f, 0f,
                            0f, 0f, 0f, 0f,
                            0f, 0f, 0f, 0f);
    }

    public static Matrix4f identityMatrix ()
    {
        return new Matrix4f(1f, 0f, 0f, 0f,
                            0f, 1f, 0f, 0f,
                            0f, 0f, 1f, 0f,
                            0f, 0f, 0f, 1f);
    }

    public static Matrix4f angle45Matrix ()
    {
        return new Matrix4f( 0.707f,  0.500f, -0.500f,  0.000f,
                             0.000f,  0.707f,  0.707f,  0.000f,
                             0.707f, -0.500f,  0.500f,  0.000f,
                             0.000f,  0.000f,  0.000f,  1.000f);
    }

    public static Matrix4f angleViewMatrix ()
    {
        return new Matrix4f( 0.684f,  0.304f, -0.663f,  0.000f,
                            -0.000f,  0.910f,  0.414f,  0.000f,
                             0.729f, -0.282f,  0.623f,  0.000f,
                             0.000f,  0.000f,  0.000f,  1.000f);        
    }

    public float[] getMatrix()
    {
        return matrix;
    }

    public Matrix4f times (Matrix4f mat)
    {
        float[] newMat = new float[16];

        newMat[0] = this.matrix[0] * mat.matrix[0] + this.matrix[1] * mat.matrix[4] + this.matrix[2] * mat.matrix[8] + this.matrix[3] * mat.matrix[12];
        newMat[1] = this.matrix[0] * mat.matrix[1] + this.matrix[1] * mat.matrix[5] + this.matrix[2] * mat.matrix[9] + this.matrix[3] * mat.matrix[13];
        newMat[2] = this.matrix[0] * mat.matrix[2] + this.matrix[1] * mat.matrix[6] + this.matrix[2] * mat.matrix[10] + this.matrix[3] * mat.matrix[14];
        newMat[3] = this.matrix[0] * mat.matrix[3] + this.matrix[1] * mat.matrix[7] + this.matrix[2] * mat.matrix[11] + this.matrix[3] * mat.matrix[15];

        newMat[4] = this.matrix[4] * mat.matrix[0] + this.matrix[5] * mat.matrix[4] + this.matrix[6] * mat.matrix[8] + this.matrix[7] * mat.matrix[12];
        newMat[5] = this.matrix[4] * mat.matrix[1] + this.matrix[5] * mat.matrix[5] + this.matrix[6] * mat.matrix[9] + this.matrix[7] * mat.matrix[13];
        newMat[6] = this.matrix[4] * mat.matrix[2] + this.matrix[5] * mat.matrix[6] + this.matrix[6] * mat.matrix[10] + this.matrix[7] * mat.matrix[14];
        newMat[7] = this.matrix[4] * mat.matrix[3] + this.matrix[5] * mat.matrix[7] + this.matrix[6] * mat.matrix[11] + this.matrix[7] * mat.matrix[15];

        newMat[8] = this.matrix[8] * mat.matrix[0] + this.matrix[9] * mat.matrix[4] + this.matrix[10] * mat.matrix[8] + this.matrix[11] * mat.matrix[12];
        newMat[9] = this.matrix[8] * mat.matrix[1] + this.matrix[9] * mat.matrix[5] + this.matrix[10] * mat.matrix[9] + this.matrix[11] * mat.matrix[13];
        newMat[10] = this.matrix[8] * mat.matrix[2] + this.matrix[9] * mat.matrix[6] + this.matrix[10] * mat.matrix[10] + this.matrix[11] * mat.matrix[14];
        newMat[11] = this.matrix[8] * mat.matrix[3] + this.matrix[9] * mat.matrix[7] + this.matrix[10] * mat.matrix[11] + this.matrix[11] * mat.matrix[15];

        newMat[12] = this.matrix[12] * mat.matrix[0] + this.matrix[13] * mat.matrix[4] + this.matrix[14] * mat.matrix[8] + this.matrix[15] * mat.matrix[12];
        newMat[13] = this.matrix[12] * mat.matrix[1] + this.matrix[13] * mat.matrix[5] + this.matrix[14] * mat.matrix[9] + this.matrix[15] * mat.matrix[13];
        newMat[14] = this.matrix[12] * mat.matrix[2] + this.matrix[13] * mat.matrix[6] + this.matrix[14] * mat.matrix[10] + this.matrix[15] * mat.matrix[14];
        newMat[15] = this.matrix[12] * mat.matrix[3] + this.matrix[13] * mat.matrix[7] + this.matrix[14] * mat.matrix[11] + this.matrix[15] * mat.matrix[15];

        return new Matrix4f(newMat);
    }

    public Matrix4f times(float scaleFactor)
    {
        float[] newMat = new float[16];
        System.arraycopy(this.matrix, 0, newMat, 0, 16);

        newMat[0]  *= scaleFactor;
        newMat[5]  *= scaleFactor;
        newMat[10] *= scaleFactor;
        newMat[15] *= scaleFactor;

        return new Matrix4f(newMat);
    }
    
    public Point3f times(Point3f pnt)
    {
        return new Point3f(this.matrix[0] * pnt.x + this.matrix[4] * pnt.y + this.matrix[8]  * pnt.z + this.matrix[12],
                           this.matrix[1] * pnt.x + this.matrix[5] * pnt.y + this.matrix[9]  * pnt.z + this.matrix[13],
                           this.matrix[2] * pnt.x + this.matrix[6] * pnt.y + this.matrix[10] * pnt.z + this.matrix[14]);
    }

    public boolean isInverting()
    {
        if (this.matrix[5] < 0)
            return true;
        else
             return false;
    }

//    public RotationMatrix transpose ()
//    {
//        RotationMatrix newMat = new RotationMatrix();
//
//        this.matrix[0](this.matrix[0]);
//        newMat.setX1(this.matrix[1]);
//        newMat.setX2(this.matrix[2]);
//        newMat.setY0(this.matrix[4]);
//        newMat.setY1(this.matrix[5]);
//        newMat.setY2(this.matrix[6]);
//        newMat.setZ0(this.matrix[8]);
//        newMat.setZ1(this.matrix[9]);
//        newMat.setZ2(this.matrix[10]);
//
//        return newMat;
//    }
//
//    public RotationMatrix inverse ()
//    {
//        RotationMatrix newMat = new RotationMatrix();
//        RotationMatrix ludmat = this.luDecomp();
//
//        float[] col0 = ludmat.luBackSub(1.0f, 0.0, 0.0);
//        float[] col1 = ludmat.luBackSub(0.0, 1.0, 0.0);
//        float[] col2 = ludmat.luBackSub(0.0, 0.0, 1.0);
//
//        this.matrix[0](col0[0]);
//        newMat.setX1(col0[1]);
//        newMat.setX2(col0[2]);
//        newMat.setY0(col1[0]);
//        newMat.setY1(col1[1]);
//        newMat.setY2(col1[2]);
//        newMat.setZ0(col2[0]);
//        newMat.setZ1(col2[1]);
//        newMat.setZ2(col2[2]);
//
//        return newMat;
//    }
//
//    private RotationMatrix luDecomp ()
//    {
//        float big = 0.0;
//        float temp = 0.0;
//        float[] vv = new float[3];
//
//        float[][] newArray = this.toArray();
//
//        for (int this.matrix[10] = 0; this.matrix[10] < 3; this.matrix[10] ++)
//        {
//            big = 0.0D;
//            for (int j = 0; j < 3; j ++)
//                if ((temp = Math.abs(newArray[this.matrix[10]][j])) > big)
//                    big = temp;
//            if (big == 0.0D)
//            {
//                System.out.println("Error cannot perform inverse on this.matrix[0] matrix with an entire column of zeros");
//                return null;
//            }
//            vv[this.matrix[10]] = 1.0 / big;
//        }
//
//        int imax = 0;
//        float dum = 0.0;
//        float sum = 0.0;
//        float dumm = 0.0;
//        int[] permIndex = new int[3];
//        for (int j = 0; j < 3; j ++)
//        {
//            for (int this.matrix[10] = 0; this.matrix[10] < j; this.matrix[10] ++)
//            {
//                sum = newArray[this.matrix[10]][j];
//                for (int k = 0; k < this.matrix[10]; k ++)
//                    sum -= newArray[this.matrix[10]][k] * newArray[k][j];
//                newArray[this.matrix[10]][j] = sum;
//            }
//            big = 0.0;
//            for (int this.matrix[10] = j; this.matrix[10] < 3; this.matrix[10] ++)
//            {
//                sum = newArray[this.matrix[10]][j];
//                for (int k = 0; k < j; k ++)
//                    sum -= newArray[this.matrix[10]][k] * newArray[k][j];
//                newArray[this.matrix[10]][j] = sum;
//                if ((dum = vv[this.matrix[10]] * Math.abs(sum)) >= big)
//                {
//                    big = dum;
//                    imax = this.matrix[10];
//                }
//            }
//            if (j != imax)
//            {
//                for (int k = 0; k < 3; k ++)
//                {
//                    dumm = newArray[imax][k];
//                    newArray[imax][k] = newArray[j][k];
//                    newArray[j][k] = dumm;
//                }
//                vv[imax] = vv[j];
//            }
//            permIndex[j] = imax;
//
//            if (newArray[j][j] == 0.0)
//            {
//                newArray[j][j] = 1.0e-100;
//            }
//            if (j != 2)
//            {
//                dumm = 1.0 / newArray[j][j];
//                for (int this.matrix[10] = j + 1; this.matrix[10] < 3; this.matrix[10] ++)
//                {
//                    newArray[this.matrix[10]][j] *= dumm;
//                }
//            }
//        }
//
//        RotationMatrix newMat = from_Array(newArray);
//        newMat.permutationIndex = permIndex;
//        return newMat;
//    }
//
//    private float[] luBackSub (float this.matrix[12], float this.matrix[13], float this.matrix[14])
//    {
//        float temp = 0.0;
//        float[] xvec = {this.matrix[12], this.matrix[13], this.matrix[14]};
//
//        temp = xvec[permutationIndex[0]];
//        xvec[permutationIndex[0]] = xvec[0];
//        xvec[0] = temp;
//
//        temp = xvec[permutationIndex[1]];
//        xvec[permutationIndex[1]] = xvec[1];
//        xvec[1] = temp - this.matrix[4] * xvec[0];
//
//        temp = xvec[permutationIndex[2]];
//        xvec[permutationIndex[2]] = xvec[2];
//        xvec[2] = temp - this.matrix[8] * xvec[0] - this.matrix[9] * xvec[1];
//
//        xvec[2] = xvec[2] / this.matrix[10];
//        xvec[1] = (xvec[1] - this.matrix[6] * xvec[2]) / this.matrix[5];
//        xvec[0] = (xvec[0] - this.matrix[2] * xvec[2] - this.matrix[1] * xvec[1]) / this.matrix[0];
//
//        return xvec;
//    }

    public Matrix4f copy()
    {
        return new Matrix4f(this.matrix[0], this.matrix[1], this.matrix[2], this.matrix[3], this.matrix[4], this.matrix[5], this.matrix[6], this.matrix[7], this.matrix[8], this.matrix[9], this.matrix[10], this.matrix[11], this.matrix[12], this.matrix[13], this.matrix[14], this.matrix[15]);
    }

    @Override
    public String toString ()
    {
        StringBuilder sb = new StringBuilder();
        sb.append("\n");
        sb.append(this.matrix[0]).append(" ").append(this.matrix[1]).append(" ").append(this.matrix[2]).append(" ").append(this.matrix[3]).append("\n");
        sb.append(this.matrix[4]).append(" ").append(this.matrix[5]).append(" ").append(this.matrix[6]).append(" ").append(this.matrix[7]).append("\n");
        sb.append(this.matrix[8]).append(" ").append(this.matrix[9]).append(" ").append(this.matrix[10]).append(" ").append(this.matrix[11]).append("\n");
        sb.append(this.matrix[12]).append(" ").append(this.matrix[13]).append(" ").append(this.matrix[14]).append(" ").append(this.matrix[15]).append("\n");

        return sb.toString();
    }
}
