
package com.locad.geometry;

public class Point3f
{
    public float x = Float.NaN;
    public float y = Float.NaN;
    public float z = Float.NaN;

    public Point3f (float x, float y, float z)
    {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    public Point3f (Point3f point)
    {
        this.x = point.getX();
        this.y = point.getY();
        this.z = point.getZ();
    }

    public float[] toArray()
    {
        float[] array = new float[3];
        array[0] = x;
        array[1] = y;
        array[2] = z;
        return array;
    }

    public float[][] toArray2D()
    {
        float[][] array = new float[3][1];
        array[0][0] = x;
        array[1][0] = y;
        array[2][0] = z;
        return array;
    }

    public Point3f min(Point3f p)
    {
        Point3f newP = this.copy();
        if (p.x < newP.x)
        {
            newP.x = p.x;
        }
        if (p.y < newP.y)
        {
            newP.y = p.y;
        }
        if (p.z < newP.z)
        {
            newP.z = p.z;
        }
        return newP;
    }

    public Point3f max(Point3f p)
    {
        Point3f newP = this.copy();
        if (p.x > newP.x)
        {
            newP.x = p.x;
        }
        if (p.y > newP.y)
        {
            newP.y = p.y;
        }
        if (p.z > newP.z)
        {
            newP.z = p.z;
        }
        return newP;
    }

    public Point3f div(float a)
    {
        return new Point3f(this.x / a, this.y / a, this.z / a);
    }

    public Point3f times(float a)
    {
        return new Point3f(this.x * a, this.y * a, this.z * a);
    }

    public Point3f plus(Point3f p)
    {
        return new Point3f(this.x + p.x, this.y + p.y, this.z + p.z);
    }

    public Point3f minus(Point3f p)
    {
        return new Point3f(this.x - p.x, this.y - p.y, this.z - p.z);
    }

    public Point3f crossProd (Point3f p)
    {
        return new Point3f(this.y*p.z - this.z*p.y, this.z*p.x - this.x*p.z, this.x*p.y - this.y*p.x);
    }

    public float dotProd(Point3f p)
    {
        return this.x * p.x + this.y * p.y + this.z * p.z;
    }

    public float mag()
    {
        return (float) Math.sqrt(this.x*this.x + this.y*this.y + this.z*this.z);
    }

    public Point3f unitVector()
    {
        return this.div(this.mag());
    }

    public float distanceTo(Point3f p)
    {
        return this.minus(p).mag();
    }

    public float getAngleFromAnotherVector (Point3f B)
    {
        float dotProduct = this.dotProd(B);
        float denominator = this.mag() * B.mag();
        float product = denominator != 0f ? dotProduct / denominator : 0f;
        return (float) Math.acos(product);
    }

    public float getX ()
    {
        return x;
    }

    public void setX (float posX)
    {
        this.x = posX;
    }

    public float getY ()
    {
        return y;
    }

    public void setY (float posY)
    {
        this.y = posY;
    }

    public float getZ ()
    {
        return z;
    }

    public void setZ (float posZ)
    {
        this.z = posZ;
    }

    public Point3f copy ()
    {
        return new Point3f(this.x, this.y, this.z);
    }

    @Override
    public String toString ()
    {
        return x + ", " + y + ", " + z;
    }
}
