
package com.locad.geometry;

import com.locad.enumerations.Enumerations.MaterialTypes;
import com.locad.enumerations.Enumerations.ViewAngleModes;
import com.locad.model.Color4b;
import javax.media.opengl.GL;
import org.apache.log4j.Logger;

public class Quadrilateral extends Shape
{
    private static final Logger log = Logger.getLogger(Quadrilateral.class);
    
    public final Point3f p1;
    public final Point3f p2;
    public final Point3f p3;
    public final Point3f p4;
    public final Point3f norm;    

    public Quadrilateral(Point3f p1, Point3f p2, Point3f p3, Point3f p4, Point3f norm, short colorCode)
    {
        this.p1 = p1;
        this.p2 = p2;
        this.p3 = p3;
        this.p4 = p4;
        this.norm = norm;
        this.colorCode = colorCode;
    }

    public static Quadrilateral from_Line (String input, boolean BFCFlag)
    {
        String[] tokens = input.trim().split(" ");
        String[] t = new String[tokens.length];
        int ndx = 0;
        for (int i=0; i<tokens.length; i++)
        {
            if (tokens[i] == null ? "" != null : ! tokens[i].equals(""))
            {
                t[ndx] = tokens[i];
                ndx++;
            }
        }

        Point3f p1;
        Point3f p2;
        Point3f p3;
        Point3f p4;

        if (BFCFlag)
        {
            p4 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            p3 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));
            p2 = new Point3f(new Float(t[8]), new Float(t[9]), new Float(t[10]));
            p1 = new Point3f(new Float(t[11]), new Float(t[12]), new Float(t[13]));
        }
        else
        {
            p1 = new Point3f(new Float(t[2]), new Float(t[3]), new Float(t[4]));
            p2 = new Point3f(new Float(t[5]), new Float(t[6]), new Float(t[7]));
            p3 = new Point3f(new Float(t[8]), new Float(t[9]), new Float(t[10]));
            p4 = new Point3f(new Float(t[11]), new Float(t[12]), new Float(t[13]));
        }
        
        Point3f norm = getNormal(p1, p2, p3);

        // sometimes, the quadrilateral points are not in the correct order
        double signedArea123 = getSignedArea(p1, p2, p3, norm);
        if (signedArea123 > 0)
        {
            double signedArea134 = getSignedArea(p1, p3, p4, norm);      
            if (signedArea134 > 0)
            {
                // no swap needed for clockwise order       
            }
            else
            {
                double signedArea124 = getSignedArea(p1, p2, p4, norm);
                
                if (signedArea124 > 0)
                {
                    Point3f temp = p3.copy();
                    p3 = p4.copy();
                    p4 = temp;                  
                }
                else
                {
                    Point3f temp = p1.copy();
                    p1 = p4.copy();
                    p4 = temp;            
                }
            }    
        }
        else
        {
            double signedArea134 = getSignedArea(p1, p3, p4, norm);
            if (signedArea134 < 0)
            {
                Point3f temp = p2.copy();
                p2 = p4.copy();
                p4 = temp;      
            }
            else
            {
                double signedArea124 = getSignedArea(p1, p2, p4, norm);
                if (signedArea124 > 0)
                {
                    Point3f temp = p2.copy();
                    p2 = p3.copy();
                    p3 = temp;                  
                }
                else
                {
                    Point3f temp = p1.copy();
                    p1 = p2.copy();
                    p2 = temp;            
                }
            }             
        }
        
        return new Quadrilateral(p1, p2, p3, p4, norm, new Short(t[1]));
    }

    private static Point3f getNormal(Point3f p1, Point3f p2, Point3f p3)
    {
        // Calculate The Vector From Point 1 To Point 0
        Point3f v1 = new Point3f(p1.getX() - p2.getX(), p1.getY() - p2.getY(), p1.getZ() - p2.getZ());

        // Calculate The Vector From Point 2 To Point 1
        Point3f v2 = new Point3f(p2.getX() - p3.getX(), p2.getY() - p3.getY(), p2.getZ() - p3.getZ());

        // Compute The Cross Product To Give Us A Surface Normal
        return v1.crossProd(v2).unitVector();
    }

    private static double getSignedArea(Point3f p1, Point3f p2, Point3f p3, Point3f normal)
    {
        // select largest abs coordinate to ignore for projection
        float ax = (normal.x > 0 ? normal.x : -normal.x);     // abs x-coord
        float ay = (normal.y > 0 ? normal.y : -normal.y);     // abs y-coord
        float az = (normal.z > 0 ? normal.z : -normal.z);     // abs z-coord
        double an = Math.sqrt(ax * ax + ay * ay + az * az);  // length of normal vector

        // project onto 2D plane and calculate area
        if (ax > ay)
        {
            if (ax > az)
            {
                return ((p2.y * (p3.z - p1.z)) + (p3.y * (p1.z - p2.z)) + (p1.y * (p2.z - p3.z))) * (an / (2 * ax));
            }
        }
        else if (ay > az)
        {
            return ((p2.x * (p3.z - p1.z)) + (p3.x * (p1.z - p2.z)) + (p1.x * (p2.z - p3.z))) * (an / (2 * ay));
        }

        return ((p2.x * (p3.y - p1.y)) + (p3.x * (p1.y - p2.y)) + (p1.x * (p2.y - p3.y))) * (an / (2 * az));
    }

    @Override
    public void addToGL(GL gl, short requestedColorCode, MaterialTypes materialType)
    {
        Color4b color = getColor(requestedColorCode, materialType);
        if (color != null)
        {
            gl.glBegin(GL.GL_POLYGON);
            gl.glColor4f((color.red + 128) / 255f, (color.green + 128) / 255f, (color.blue + 128) / 255f, (color.alpha + 128) / 255f);
            gl.glVertex3f(p1.x, p1.y, p1.z);
            gl.glVertex3f(p2.x, p2.y, p2.z);
            gl.glVertex3f(p3.x, p3.y, p3.z);
            gl.glVertex3f(p4.x, p4.y, p4.z);
            gl.glEnd();                   
        }
    }

    @Override
    public void addToGLwithBFC(GL gl, short requestedColorCode, MaterialTypes materialType, boolean BFCflag)
    {
        Color4b color = getColor(requestedColorCode, materialType);
        if (color != null)
        {
            Point3f temp1;
            Point3f temp2;
            Point3f temp3;
            Point3f temp4;

            if (BFCflag)
            {
                temp1 = (p4);
                temp2 = (p3);
                temp3 = (p2);
                temp4 = (p1);
            }
            else
            {
                temp1 = (p1);
                temp2 = (p2);
                temp3 = (p3);
                temp4 = (p4);
            }
            
            // compute the norm - this must be counter clockwise
//            Point3f edge1 = temp2.minus(temp1);
//            Point3f edge2 = temp4.minus(temp1);
//            Point3f norm = edge1.crossProd(edge2).unitVector();

            gl.glBegin(GL.GL_POLYGON);
            gl.glColor4f((color.red + 128) / 255f, (color.green + 128) / 255f, (color.blue + 128) / 255f, (color.alpha + 128) / 255f);
            //gl.glNormal3f(p1.x + norm.x, p1.y + norm.y, p1.z + norm.z);
            gl.glNormal3f(norm.x, norm.y, norm.z);
            gl.glVertex3f(temp1.x, temp1.y, temp1.z);
            gl.glVertex3f(temp2.x, temp2.y, temp2.z);
            gl.glVertex3f(temp3.x, temp3.y, temp3.z);
            gl.glVertex3f(temp4.x, temp4.y, temp4.z);
            gl.glEnd();
        }
    }

    @Override
    public void getBoundingBox (Matrix4f orientation, Point3f min, Point3f max)
    {
        Point3f temp1 = orientation.times(p1);
        Point3f temp2 = orientation.times(p2);
        Point3f temp3 = orientation.times(p3);
        Point3f temp4 = orientation.times(p4);       

        if (temp1.x < min.x)
            min.x = temp1.x;
        if (temp1.x > max.x)
            max.x = temp1.x;
        if (temp1.y < min.y)
            min.y = temp1.y;
        if (temp1.y > max.y)
            max.y = temp1.y;
        if (temp1.z < min.z)
            min.z = temp1.z;
        if (temp1.z > max.z)
            max.z = temp1.z;

        if (temp2.x < min.x)
            min.x = temp2.x;
        if (temp2.x > max.x)
            max.x = temp2.x;
        if (temp2.y < min.y)
            min.y = temp2.y;
        if (temp2.y > max.y)
            max.y = temp2.y;
        if (temp2.z < min.z)
            min.z = temp2.z;
        if (temp2.z > max.z)
            max.z = temp2.z;

        if (temp3.x < min.x)
            min.x = temp3.x;
        if (temp3.x > max.x)
            max.x = temp3.x;
        if (temp3.y < min.y)
            min.y = temp3.y;
        if (temp3.y > max.y)
            max.y = temp3.y;
        if (temp3.z < min.z)
            min.z = temp3.z;
        if (temp3.z > max.z)
            max.z = temp3.z;

        if (temp4.x < min.x)
            min.x = temp1.x;
        if (temp4.x > max.x)
            max.x = temp4.x;
        if (temp4.y < min.y)
            min.y = temp4.y;
        if (temp4.y > max.y)
            max.y = temp4.y;
        if (temp4.z < min.z)
            min.z = temp4.z;
        if (temp4.z > max.z)
            max.z = temp4.z;
    }

    @Override
    public boolean contains (Matrix4f orientation, Point3f line, ViewAngleModes viewAngle)
    {
        Point3f temp1 = orientation.times(p1);
        Point3f temp2 = orientation.times(p2);
        Point3f temp3 = orientation.times(p3);
        Point3f temp4 = orientation.times(p4);

        int crossings = 0;
        if(viewAngle == ViewAngleModes.FRONT || viewAngle == ViewAngleModes.BACK)
        {
            if (((temp1.y <= line.y) && (line.y < temp2.y) || (temp2.y <= line.y) && (line.y < temp1.y)) && line.x < ((temp2.x - temp1.x) / (temp2.y - temp1.y)) * (line.y - temp1.y) + temp1.x)
                crossings++;
            if (((temp2.y <= line.y) && (line.y < temp3.y) || (temp3.y <= line.y) && (line.y < temp2.y)) && line.x < ((temp3.x - temp2.x) / (temp3.y - temp2.y)) * (line.y - temp2.y) + temp2.x)
                crossings++;
            if (((temp3.y <= line.y) && (line.y < temp4.y) || (temp4.y <= line.y) && (line.y < temp3.y)) && line.x < ((temp4.x - temp3.x) / (temp4.y - temp3.y)) * (line.y - temp3.y) + temp3.x)
                crossings++;
            if (((temp4.y <= line.y) && (line.y < temp1.y) || (temp1.y <= line.y) && (line.y < temp4.y)) && line.x < ((temp1.x - temp4.x) / (temp1.y - temp4.y)) * (line.y - temp4.y) + temp4.x)
                crossings++;
        }
        else if(viewAngle == ViewAngleModes.LEFT || viewAngle == ViewAngleModes.RIGHT)
        {
            if (((temp1.y <= line.y) && (line.y < temp2.y) || (temp2.y <= line.y) && (line.y < temp1.y)) && line.z < ((temp2.z - temp1.z) / (temp2.y - temp1.y)) * (line.y - temp1.y) + temp1.z)
                crossings++;
            if (((temp2.y <= line.y) && (line.y < temp3.y) || (temp3.y <= line.y) && (line.y < temp2.y)) && line.z < ((temp3.z - temp2.z) / (temp3.y - temp2.y)) * (line.y - temp2.y) + temp2.z)
                crossings++;
            if (((temp3.y <= line.y) && (line.y < temp4.y) || (temp4.y <= line.y) && (line.y < temp3.y)) && line.z < ((temp4.z - temp3.z) / (temp4.y - temp3.y)) * (line.y - temp3.y) + temp3.z)
                crossings++;
            if (((temp4.y <= line.y) && (line.y < temp1.y) || (temp1.y <= line.y) && (line.y < temp4.y)) && line.z < ((temp1.z - temp4.z) / (temp1.y - temp4.y)) * (line.y - temp4.y) + temp4.z)
                crossings++;
        }
        else if(viewAngle == ViewAngleModes.TOP || viewAngle == ViewAngleModes.BOTTOM)
        {
            if (((temp1.z <= line.z) && (line.z < temp2.z) || (temp2.z <= line.z) && (line.z < temp1.z)) && line.x < ((temp2.x - temp1.x) / (temp2.z - temp1.z)) * (line.z - temp1.z) + temp1.x)
                crossings++;
            if (((temp2.z <= line.z) && (line.z < temp3.z) || (temp3.z <= line.z) && (line.z < temp2.z)) && line.x < ((temp3.x - temp2.x) / (temp3.z - temp2.z)) * (line.z - temp2.z) + temp2.x)
                crossings++;
            if (((temp3.z <= line.z) && (line.z < temp4.z) || (temp4.z <= line.z) && (line.z < temp3.z)) && line.x < ((temp4.x - temp3.x) / (temp4.z - temp3.z)) * (line.z - temp3.z) + temp3.x)
                crossings++;
            if (((temp4.z <= line.z) && (line.z < temp1.z) || (temp1.z <= line.z) && (line.z < temp4.z)) && line.x < ((temp1.x - temp4.x) / (temp1.z - temp4.z)) * (line.z - temp4.z) + temp4.x)
                crossings++;
        }

        return crossings % 2 != 0;
    }

    @Override
    public Quadrilateral copy()
    {
        return new Quadrilateral(this.p1, this.p2, this.p3, this.p4, this.norm, this.colorCode);
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append("shapeType ").append("Quadrilateral").append("\n");
        strBuffer.append(super.toString());
        strBuffer.append("p1 ").append(p1).append("\n");
        strBuffer.append("p2 ").append(p2).append("\n");
        strBuffer.append("p3 ").append(p3).append("\n");
        strBuffer.append("p4 ").append(p4).append("\n");

        return strBuffer.toString();
    }

    @Override
    public String toString (Matrix4f orientation)
    {
        Point3f temp1 = orientation.times(p1);
        Point3f temp2 = orientation.times(p2);
        Point3f temp3 = orientation.times(p3);
        Point3f temp4 = orientation.times(p4);

        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append("shapeType ").append("Quadrilateral").append("\n");
        strBuffer.append(super.toString());
        strBuffer.append("p1 ").append(temp1).append("\n");
        strBuffer.append("p2 ").append(temp2).append("\n");
        strBuffer.append("p3 ").append(temp3).append("\n");
        strBuffer.append("p4 ").append(temp4).append("\n");

        return strBuffer.toString();
    }
}