
package com.locad.ldraw;

import com.locad.enumerations.Enumerations.MaterialTypes;
import com.locad.main.Configuration;
import com.locad.model.Color4b;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;
import org.apache.log4j.Logger;

public final class ColorLibrary
{
    private final Logger log;
    
    public Map<Short, LDrawColor> colors = new HashMap<Short, LDrawColor>();

    private static ColorLibrary instance = null;

    public static ColorLibrary getInstance()
    {
        synchronized (ColorLibrary.class)
        {
            if (instance == null)
            {
                instance = new ColorLibrary();
            }
        }
        return instance;
    }

    public ColorLibrary()
    {
        log = Logger.getLogger(getClass().getName());
    }

    public void readLDConfig()
    {
        try
        {
            Scanner s = new Scanner(new BufferedReader(new FileReader(Configuration.getInstance().colorFile)));

            while (s.hasNextLine())
            {
                String[] tokens = s.nextLine().trim().split(" ");
                String[] t = new String[tokens.length];
                int ndx = 0;
                for (int i=0; i<tokens.length; i++)
                {
                    if (tokens[i] == null ? "" != null : ! tokens[i].equals(""))
                    {
                        t[ndx] = tokens[i];
                        ndx++;
                    }
                }
                if (t.length >= 9)
                {
                    if (t[0].equals("0") && t[1].equals("!COLOUR"))
                    {
                        Color4b edgeColor;
                        Color4b fillColor;
                        int r, g, b;
                        int eColor;
                        byte luminance;
                        MaterialTypes materialType = MaterialTypes.PLASTIC;
                        if (t[8].startsWith("#"))
                        {
                            eColor = Integer.valueOf(t[8].substring(1), 16);
                            r = (eColor >> 16) & 0xFF;
                            g = (eColor >> 8) & 0xFF;
                            b = eColor & 0xFF;
                            edgeColor = new Color4b((byte) (r - 128), (byte) (g - 128), (byte) (b - 128), (byte) 127);
                        }
                        else
                        {
                            edgeColor = new Color4b((byte) -128, (byte) -128, (byte) -128, (byte) 127);
                        }
                        eColor = Integer.valueOf(t[6].substring(1), 16);
                        r = (eColor >> 16) & 0xFF;
                        g = (eColor >> 8) & 0xFF;
                        b = eColor & 0xFF;
                        fillColor = new Color4b((byte) (r - 128), (byte) (g - 128), (byte) (b - 128), (byte) 127);
                        luminance = 0;
                        for (int i=9; i<ndx; i++)
                        {
                            if (ndx >= i+1)
                            {
                                if (t[i].equalsIgnoreCase("ALPHA"))
                                {
                                    int a = Integer.valueOf(t[10]);
                                    fillColor = new Color4b((byte) (r - 128), (byte) (g - 128), (byte) (b - 128), (byte) (a - 128));
                                }
                                if (t[i].equalsIgnoreCase("LUMINANCE"))
                                {
                                    luminance = Byte.valueOf(t[12]);
                                    materialType = MaterialTypes.LUMINANT;
                                }
                                if (t[i].equalsIgnoreCase("RUBBER"))
                                {
                                    materialType = MaterialTypes.RUBBER;
                                }
                                if (t[i].equalsIgnoreCase("METAL"))
                                {
                                    materialType = MaterialTypes.METAL;
                                }
                                if (t[i].equalsIgnoreCase("PEARLESCENT"))
                                {
                                    materialType = MaterialTypes.PEARLESCENT;
                                }
                                if (t[i].equalsIgnoreCase("CHROME"))
                                {
                                    materialType = MaterialTypes.CHROME;
                                }
                            }
                        }
                        
                        if (fillColor.alpha < 120)
                            materialType = MaterialTypes.TRANSPARENT;

                        colors.put(Short.valueOf(t[4]), new LDrawColor(t[2], fillColor, edgeColor, luminance, materialType));
                    }
                }
            }
            s.close();
        }
        catch (FileNotFoundException ex)
        {
            log.error("error " + ex);
        }
    }

    public LDrawColor getColor(short code)
    {
        if (colors.containsKey(code))
        {
            return colors.get(code);
        }
        else
        {
            for (Short key : colors.keySet())
            {
                if (colors.get(key).name.equalsIgnoreCase("WHITE"))
                {
                    return colors.get(key);
                }
            }
            return null;
        }
    }

    @Override
    public String toString()
    {
        StringBuilder strBuffer = new StringBuilder();

        for (Short key : colors.keySet())
        {
            strBuffer.append("ColorCode: ").append(key).append(colors.get(key).toString());
        }

        return strBuffer.toString();
    }
}
