
package com.locad.main;

import com.locad.ldraw.ColorLibrary;
import com.locad.ldraw.PartsLibrary;
import com.locad.ldraw.Project;
import com.locad.model.Model;
import java.awt.EventQueue;
import java.io.File;
import javax.swing.UIManager;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

public final class Main
{
    private static final Logger log = Logger.getLogger(Main.class);

    public Main()
    {
        // make sure configuration has been created
        Configuration config = Configuration.getInstance();

        // create color library
        ColorLibrary.getInstance().readLDConfig();

        // create parts library
        PartsLibrary.getInstance().readPartFiles();

//        log.info("Enabling OpenGL-Swing pipe");
//        System.setProperty("sun.java2d.opengl", "true");

        Desktop desktop = Desktop.getInstance();
        desktop.setVisible(true);
        desktop.postInit();
    }

    public static void main(String[] args)
    {
        if (Configuration.getInstance().keepLogs)
            LoggingUtils.setupLog4j("LOCAD.log");
        else
            DOMConfigurator.configureAndWatch("config/logger.xml", 500);
        log.info("LEGO Open CAD started");

        // Run this in the AWT event thread to prevent deadlocks and race conditions
        EventQueue.invokeLater(new Runnable()
        {
            @Override
            public void run()
            {
                try
                {
                    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                }
                catch (Exception ex)
                {
                    log.warn("Can not enable look and feel " + ex);
                }
            }
        });

        Main m = new Main();
        
        Project project = Project.from_File(new File(Configuration.getInstance().projectFile), Configuration.getInstance().defaultColorCode);
        Model.getInstance().merge(project);
    }
}
