
package com.locad.viewport;

import com.locad.enumerations.Enumerations.ViewAngleModes;
import com.locad.ldraw.Project;
import com.locad.main.Configuration;
import com.sun.opengl.util.FPSAnimator;
import javax.media.opengl.GLCapabilities;
import javax.media.opengl.GLJPanel;

public final class Canvas extends GLJPanel
{
    private final Camera camera;
    private final Renderer renderer;
    private final Listener listener;
    private final FPSAnimator animator;

    public Canvas(ViewAngleModes viewAngle)
    {
        super(createGLCapabilites());

        this.camera = new Camera(viewAngle);
        this.renderer = new Renderer(camera, viewAngle);
        this.listener = new Listener(this, renderer, camera);
        this.animator = new FPSAnimator(this, 30);
    }

    public Canvas(Project project, ViewAngleModes viewAngle)
    {
        super(createGLCapabilites());

        this.camera = new Camera(viewAngle);
        this.renderer = new Renderer(project, camera, viewAngle);
        this.listener = new Listener(this, renderer, camera);
        this.animator = new FPSAnimator(this, 20);
    }

    public void postInit()
    {
        this.addGLEventListener(renderer);
        //this.addKeyListener(listener);
        this.addMouseListener(listener);
        this.addMouseMotionListener(listener);
        this.addMouseWheelListener(listener);
        this.repaint();
    }

    public void setViewAngle(ViewAngleModes viewAngle)
    {
        renderer.setViewAngle(viewAngle);
    }

    public void setProject(Project project)
    {
        this.renderer.setProject(project);
        this.display();
    }

    public void update()
    {
        startAnimating();
        try
        {
            Thread.sleep(10);
        }
        catch (InterruptedException ex)
        {
        }
        stopAnimating();
    }

    public void stopAnimating()
    {
        new Thread(new Runnable()
        {
            @Override
            public void run()
            {
                if (animator.isAnimating())
                {
                    animator.stop();
                }
            }
        }).start();
    }

    public void startAnimating()
    {
        new Thread(new Runnable()
        {
            @Override
            public void run()
            {
                if (!animator.isAnimating())
                {
                    animator.start();
                }
            }
        }).start();
    }

    private static GLCapabilities createGLCapabilites()
    {
        GLCapabilities capabilities = new GLCapabilities();
        capabilities.setHardwareAccelerated(true);
        capabilities.setDoubleBuffered(true);

        // try to enable 2x anti aliasing - should be supported on most hardware
        if (Configuration.getInstance().enableLighting)
        {
            capabilities.setNumSamples(2);
            capabilities.setSampleBuffers(true);
        }

        return capabilities;
    }
}
