
package com.locad.viewport;

import java.awt.event.*;
import org.apache.log4j.Logger;

public class Listener implements KeyListener, MouseListener, MouseMotionListener, MouseWheelListener
{
    private final Logger log;

    private final Canvas canvas;
    private final Renderer renderer;
    private final Camera camera;

    private boolean ctrl = false;

    public Listener (Canvas canvas, Renderer renderer, Camera camera)
    {
        log = Logger.getLogger(getClass().getName());
        
        this.canvas = canvas;
        this.renderer = renderer;
        this.camera = camera;
    }

    public void keyPressed(KeyEvent e)
    {
        log.info("yeah");
        if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
        {
            System.exit(0);     // ESC for exit
        }
        if (e.getKeyCode() == KeyEvent.VK_CONTROL)
        {
            log.info("yeah");
            ctrl = true;
        }
    }

    public void keyReleased(KeyEvent e)
    {
        if (e.getKeyCode() == KeyEvent.VK_CONTROL)
        {
            ctrl = false;
        }
    }

    public void keyTyped(KeyEvent e)
    {
        log.info("booh");
    }

    public void mouseEntered(MouseEvent e)
    {
        canvas.startAnimating();
    }

    public void mouseExited(MouseEvent e)
    {
        canvas.stopAnimating();
    }

    public void mousePressed(MouseEvent e)
    {
        camera.startDrag(e.getPoint());
        
        // LEFT BUTTON
        if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
        {
            renderer.doSelection(e.getPoint(), true);
        }

        // RIGHT BUTTON
        if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0)
        {

        }
    }

    public void mouseReleased(MouseEvent e)
    {
    }

    public void mouseClicked(MouseEvent e)
    {
        // LEFT BUTTON
        if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
        {
            renderer.doSelection(e.getPoint(), !ctrl);
        }
    }

    public void mouseDragged(MouseEvent e)
    {
        // LEFT BUTTON
        if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
        {
            camera.rotate(e.getPoint());
            renderer.doSelection(e.getPoint(), false);
        }

        // MIDDLE BUTTON
        if ((e.getModifiers() & MouseEvent.BUTTON2_MASK) != 0)
        {
            camera.pan(e.getPoint());
        }
    }

    public void mouseWheelMoved (MouseWheelEvent e)
    {
        if (e.getScrollType() == MouseWheelEvent.WHEEL_UNIT_SCROLL)
        {
            camera.zoom(e.getScrollAmount(), e.getWheelRotation());
        }
    }

    // Passive motion callback to capture mouse movements while buttons are not pressed
    public void mouseMoved(MouseEvent e)
    {
    }
}