
package com.locad.viewport;

import com.locad.enumerations.Enumerations.MaterialTypes;
import javax.media.opengl.GL;

public class Material
{
    private final float[] emmissive;
    private final float[] ambient;
    private final float[] diffuse;
    private final float[] specular;
    private final float shininess;

    public Material(float[] ambient, float[] diffuse, float[] specular, float shininess, float[] emmissive)
    {
        this.ambient = ambient;
        this.diffuse = diffuse;
        this.specular = specular;
        this.shininess = shininess;
        this.emmissive = emmissive;
    }

    public void ApplyToGL(GL gl)
    {
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, ambient, 0);
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, diffuse, 0);
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, specular, 0);
        gl.glMaterialf(GL.GL_FRONT, GL.GL_SHININESS, shininess);
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_EMISSION, emmissive, 0);
    }

    public static void setMaterialBFC(MaterialTypes materialType, GL gl)
    {
        if (materialType == MaterialTypes.LUMINANT)
        {
            gl.glEnable(GL.GL_LIGHTING);
            gl.glEnable(GL.GL_BLEND);
            gl.glEnable(GL.GL_COLOR_MATERIAL);
            Material material = new Material(new float[] {0.2f, 0.2f, 0.2f, 1.0f}, new float[] {-0.2f, -0.2f, -0.2f, 1.0f}, new float[] {-0.2f, -0.2f, -0.2f, 1.0f}, 0.1f, new float[] {0.001f, 0.001f, 0.001f, 1.0f});
            material.ApplyToGL(gl);
        }
        else if (materialType == MaterialTypes.RUBBER)
        {
            gl.glEnable(GL.GL_LIGHTING);
            gl.glDisable(GL.GL_BLEND);
            gl.glEnable(GL.GL_COLOR_MATERIAL);
            Material material = new Material(new float[] {0.2f, 0.2f, 0.2f, 1.0f}, new float[] {-0.2f, -0.2f, -0.2f, 1.0f}, new float[] {-0.2f, -0.2f, -0.2f, 1.0f}, 0.0f, new float[] {0.0f, 0.0f, 0.0f, 1.0f});
            material.ApplyToGL(gl);
        }
        else if (materialType == MaterialTypes.PLASTIC)
        {
            gl.glEnable(GL.GL_LIGHTING);
            gl.glDisable(GL.GL_BLEND);
            gl.glEnable(GL.GL_COLOR_MATERIAL);
            Material material = new Material(new float[] {0.1f, 0.1f, 0.1f, 1.0f}, new float[] {0.3f, 0.3f, 0.3f, 1.0f}, new float[] {-0.1f, -0.1f, -0.1f, 1.0f}, 0.2f, new float[] {0.0f, 0.0f, 0.0f, 1.0f});
            material.ApplyToGL(gl);
        }
        else if (materialType == MaterialTypes.GLITTER)
        {
            gl.glEnable(GL.GL_LIGHTING);
            gl.glDisable(GL.GL_BLEND);
            gl.glEnable(GL.GL_COLOR_MATERIAL);
            Material material = new Material(new float[] {0.1f, 0.1f, 0.1f, 1.0f}, new float[] {0.9f, 0.9f, 0.9f, 1.0f}, new float[] {0.9f, 0.9f, 0.9f, 1.0f}, 0.4f, new float[] {0.0f, 0.0f, 0.0f, 1.0f});
            material.ApplyToGL(gl);
        }
        else if (materialType == MaterialTypes.SPECKLE)
        {
            gl.glEnable(GL.GL_LIGHTING);
            gl.glDisable(GL.GL_BLEND);
            gl.glEnable(GL.GL_COLOR_MATERIAL);
            Material material = new Material(new float[] {0.1f, 0.1f, 0.1f, 1.0f}, new float[] {0.5f, 0.5f, 0.5f, 1.0f}, new float[] {0.7f, 0.7f, 0.7f, 1.0f}, 0.1f, new float[] {0.0f, 0.0f, 0.0f, 1.0f});
            material.ApplyToGL(gl);
        }
        else if (materialType == MaterialTypes.METAL)
        {
            gl.glEnable(GL.GL_LIGHTING);
            gl.glDisable(GL.GL_BLEND);
            gl.glEnable(GL.GL_COLOR_MATERIAL);
            Material material = new Material(new float[] {0.2f, 0.2f, 0.2f, 1.0f}, new float[] {0.8f, 0.8f, 0.8f, 1.0f}, new float[] {0.8f, 0.8f, 0.8f, 1.0f}, 0.3f, new float[] {0.0f, 0.0f, 0.0f, 1.0f});
            material.ApplyToGL(gl);
        }
        else if (materialType == MaterialTypes.PEARLESCENT)
        {
            gl.glEnable(GL.GL_LIGHTING);
            gl.glDisable(GL.GL_BLEND);
            gl.glEnable(GL.GL_COLOR_MATERIAL);
            Material material = new Material(new float[] {0.1f, 0.1f, 0.1f, 1.0f}, new float[] {0.7f, 0.7f, 0.7f, 1.0f}, new float[] {0.7f, 0.7f, 0.7f, 1.0f}, 0.2f, new float[] {0.0f, 0.0f, 0.0f, 1.0f});
            material.ApplyToGL(gl);
        }
        else if (materialType == MaterialTypes.CHROME)
        {
            gl.glEnable(GL.GL_LIGHTING);
            gl.glDisable(GL.GL_BLEND);
            gl.glEnable(GL.GL_COLOR_MATERIAL);
            Material material = new Material(new float[] {0.3f, 0.3f, 0.3f, 1.0f}, new float[] {0.9f, 0.9f, 0.9f, 1.0f}, new float[] {0.9f, 0.9f, 0.9f, 1.0f}, 0.9f, new float[] {0.0f, 0.0f, 0.0f, 1.0f});
            material.ApplyToGL(gl);
        }
        else if (materialType == MaterialTypes.TRANSPARENT)
        {
            gl.glEnable(GL.GL_LIGHTING);
            gl.glEnable(GL.GL_BLEND);
            gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
            gl.glEnable(GL.GL_COLOR_MATERIAL);
            Material material = new Material(new float[] {0.1f, 0.1f, 0.1f, 1.0f}, new float[] {0.3f, 0.3f, 0.3f, 1.0f}, new float[] {0.3f, 0.3f, 0.3f, 1.0f}, 0.1f, new float[] {0.0f, 0.0f, 0.0f, 1.0f});
            material.ApplyToGL(gl);
        }
    }

    public static void setMaterial (MaterialTypes materialType, GL gl)
    {
        if (materialType == MaterialTypes.TRANSPARENT)
        {
            gl.glDisable(GL.GL_LIGHTING);
            gl.glEnable(GL.GL_BLEND);
            gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
        }
        else
        {
            gl.glDisable(GL.GL_LIGHTING);
            gl.glDisable(GL.GL_BLEND);
        }
    }

    @Override
    public String toString ()
    {
        StringBuilder strBuffer = new StringBuilder();

        strBuffer.append("ambient ").append(ambient).append("\n");
        strBuffer.append("diffuse ").append(diffuse).append("\n");
        strBuffer.append("specular ").append(specular).append("\n");
        strBuffer.append("shininess ").append(shininess).append("\n");
        strBuffer.append("emmissive ").append(emmissive).append("\n");

        return strBuffer.toString();
    }
}
